#!/bin/bash

# Used as the shell for the maintenance user.
#
# Prevents remote command execution other than scp and sftp.
# Limits scp/sftp access to the maintenance user's chroot jail.
#
# Only allows interactive access to the maintenance console.
#
# For example, you cannot do any of these from a remote machine:
#     -  scp maintenanceuser@vapphost:/etc/file file
#     -  scp file maintenanceuser@vapphost:/etc/file
#     -  sftp maintenanceuser@vapphost:/etc/file file
#     -  ssh maintenanceuser@vapphost ls

# Prevent err msgs from getopts. We only care about the -c option
OPTERR=0

# Get the command to execute (from the -c option)
# Note that ssh will pass a command to execute as the -c option when ssh invokes this shell.
# When scp is used from a remote machine, then the command passed starts with scp.
COMMAND=
while getopts "c:" opt; do
    case $opt in
        c ) COMMAND=$OPTARG
            ;;
    esac
done

if [ "$SHLVL" -eq 1 ] ; then
    # SHELL LEVEL will be 1 when we are a login shell
    # For example when logging in via ssh, the console, or 'su - <maintenanceuser>'

    if [ -z "$COMMAND" ] ; then
        # If no command was specified, just startup an interactive shell
        exec /bin/rbash --login "$@"
    elif [[ "$COMMAND" == scp* || "$COMMAND" == *sftp-server ]]; then
        # Run the command in the maintenance user's chroot jail.
        sudo /home/maint/scripts/enter-maintenance-user-jail.sh "$COMMAND"
    else
        # Give a generic error for all other commands.
        echo "Only scp and sftp are allowed." 1>&2
        exit 1
    fi
else

    # SHELL LEVEL will be greater than 1 when we are not a login shell
    # For example when root does 'su <maintenanceuser>'
    cd ~
    exec /bin/rbash "$@"

fi

