#!/usr/bin/perl

# $Id:  $
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.


use strict;

use Fcntl ':flock'; # Import LOCK_* constants
use File::Temp qw/ tempfile tempdir /;
use File::Copy;
use File::Compare qw/ compare_text /;
use POSIX qw( strftime );
use Sys::Syslog;

use NetConfig::Interfaces;
use NetConfig::Ifconfig;
use Data::Dumper;


=head1 NAME

Net::Config - Perl module for configuring the network on this host.

=head1 DESCRIPTION

This module is used by the network-config CLI, but it can also
be used directly by other perl code.

=head1 SUBROUTINES

=over 4

=cut

# Directory in which each 'live' config file exists.
# 'live' - meaning where the OS looks for the configuration
my %liveDirs = (
    INTERFACES => "/etc/network",
    HOSTNAME => "/etc",
    HOSTS => "/etc",
    SYSCTL => "/etc",
);

# Filenames of each config file
my %files = (
    INTERFACES => "interfaces",
    HOSTNAME => "hostname",
    HOSTS => "hosts",
    SYSCTL => "sysctl.conf",
);

# subdirectory within the edit session directory where backups of original files are kept
my $backupSubdir = "backup";
# subdirectory within the edit session directory where edited config files are kept
my $editSubdir = "edit";

=item B<edit_begin>

    Function   : Begins an edit session
    Returns    : Unique session id (sid) to be used to identify the edit session, which is
               : used as input to other subroutines requiring an edit session.
    Throws     : exception if cannot create temp files for edit session

=cut
sub edit_begin {
    my $sid = tempdir();

    my ($backupDir, $editDir) = get_session_dirs($sid);

    mkdir $backupDir;
    mkdir $editDir;

    foreach my $fileId ( keys %files ) {
        copy("$liveDirs{$fileId}/$files{$fileId}", "$backupDir/$files{$fileId}") or die "Cannot create temp files for edit session: $!";
        copy("$backupDir/$files{$fileId}", "$editDir/$files{$fileId}") or die "Cannot create temp files for edit session: $!";
    }

    log_info_with_sid($sid, "begin");

    return $sid;
}


=item B<edit_status>

    Function   : Detect if edits have been made in this edit session
    Parameters : sid
    Returns    : 0 if no edits have been made in this edit session
               : 2 if edits have been made
               : 3 if edit session does not exist

=cut
sub edit_status {
    my ($sid) = @_;

    if (! sid_exists($sid)) {
        return 3;
    }

    my ($backupDir, $editDir) = get_session_dirs($sid);

    foreach my $fileId ( keys %files ) {
        compare_text("$editDir/$files{$fileId}", "$backupDir/$files{$fileId}") and return 2;
    }
    return 0;
}


=item B<edit_commit>

    Function   : Commits an edit session
    Parameters : sid
               : skipRefresh (true if the live system should NOT be refreshed after the persistent files are updated.  default is false)
               : previewOnly (when true, do not commit, just return whether or not commit would succeed)
    Throws     : exception if cannot commit files
    Returns    : 0 if commit succeeds
               : 2 if changes were made outside of the edit session

=cut
sub edit_commit {
    my ($sid, $skipRefresh, $previewOnly) = @_;
    verify_sid($sid);

    my $lockfile = '/tmp/network-config.commit.lock';

    # Use a lock file to ensure two commits don't happen at the same time.
    local *FH;
    open(FH, '>>', $lockfile) or die "Could not open '$lockfile' - $!";
    flock(FH, LOCK_EX) or die "Could not lock '$lockfile' - $!";

    if (!have_live_files_changed_since_edit_session_started($sid)) {

        if (! $previewOnly) {
            # No conflicting changes, go ahead and commit.

            my $timestamp = strftime("%Y-%m-%d_%H%M%S", localtime);

            if (!$skipRefresh) {
                stop_live_system($sid);
            }

            # Copy the edited files to the live system
            # (also save a backup copy for later debugging if necessary)

            my ($backupDir, $editDir) = get_session_dirs($sid);
            foreach my $fileId ( keys %files ) {
                # Save a copy of the backup file (this cannot be recovered automatically, but can be referenced manually)
                copy("$backupDir/$files{$fileId}", "$liveDirs{$fileId}/$files{$fileId}.$timestamp") or die "Cannot commit files: $!";

                # copy the edited files to the live filesystem
                copy("$editDir/$files{$fileId}", "$liveDirs{$fileId}/$files{$fileId}") or die "Cannot commit files: $!";
            }

            if (!$skipRefresh) {
                my $oldHostname = hostname_get();
                my $newHostname = hostname_get($sid);

                # Now start the live system with the new configuration

                system("sysctl -p");
                
                if (!start_live_system($sid, $newHostname)) {
                    # Crap!  Something messed up.  Try to restore the backup configuration

                    log_warn_with_sid($sid, "Failure occurred while applying changes.  Restoring back to previous configuration.");

                    stop_live_system($sid);

                    # Restore the backup files back to the live config.
                    foreach my $fileId ( keys %files ) {
                        copy("$backupDir/$files{$fileId}", "$liveDirs{$fileId}/$files{$fileId}") or die "Cannot restore files: $!";
                    }
                    edit_rollback($sid);

                    system("sysctl -p");

                    if (start_live_system($sid, $oldHostname)) {
                        log_warn_with_sid($sid, "Previous configuration restored.");
                    } else {
                        log_warn_with_sid($sid, "Previous configuration could not be restored.  Giving up.");
                    }
                    die "Commit failed.\n";
                }
            }

            delete_sid($sid);
            log_info_with_sid($sid, "commit");
        }
        close(FH) or die "Could not write '$lockfile' - $!";
        system("/etc/network/if-up.d/loginmsg");
        system("/etc/network/if-up.d/hosts-file");
        return 0;

    } else {
        if (! $previewOnly) {
            # Conflicting changes, rollback
            edit_rollback($sid);
        }
        close(FH) or die "Could not write '$lockfile' - $!";
        return 2;
    }

}

sub stop_live_system {
    my ($sid) = @_;
    return run_command_and_warn_on_failure($sid, "/etc/init.d/networking", "stop");
}

sub start_live_system {
    my ($sid, $hostname) = @_;
    my $success = run_command_and_warn_on_failure($sid, "/bin/hostname", $hostname);
    if ($success) {
        $success = run_command_and_warn_on_failure($sid, "/etc/init.d/networking", "start");
    }
    #Removing vmware-tools restart as vmware tools directory doesn't exist
    #if ($success) {
    #    $success = run_command_and_warn_on_failure($sid, "/etc/init.d/vmware-tools", "restart");
    #}
    return $success;
}

sub run_command_and_warn_on_failure {
    my $sid = shift;
    my ($exitCode, $output) = run_command(@_);
    # If /etc/init.d/networking stop fails, then unfortunately the exit code is still 0.  Have to look for "failed" text
    if ($exitCode != 0 || $output =~ /failed/) {
        log_warn_with_sid($sid, "Command " . join(' ', @_) . " failed ($exitCode).  Output:\n$output");
        return 0;
    }
    return 1;
}


sub run_command {

    my $command = join ' ', @_;
    reverse ($_ = qx{$command 2>&1}, $? >> 8);
}


=item B<have_live_files_changed_since_edit_session_started>

    Function   : Returns true if live system files have changed since the edit session started.
    Parameters : sid
    Returns    : 1 live system files have changed since the edit session started. 0 otherwise

=cut
sub have_live_files_changed_since_edit_session_started {
    my ($sid) = @_;
    verify_sid($sid);

    my ($backupDir, $editDir) = get_session_dirs($sid);

    foreach my $fileId ( keys %files ) {
        compare_text("$backupDir/$files{$fileId}", "$liveDirs{$fileId}/$files{$fileId}") and return 1;
    }
}

=item B<edit_rollback>

    Function   : Rolls back an edit session
    Parameters : sid
    Throws     : exception if cannot commit files, or original configuration changed since edit session started

=cut
sub edit_rollback {
    my ($sid) = @_;
    delete_sid($sid);
    log_info_with_sid($sid, "rollback");
}

=item B<interface_list>

    Function   : Lists the interfaces available to be configured.
    Parameters : none
    Returns    : list of interface names that can be configured

=cut
sub interface_list {

    my $ifconfigModel = Ifconfig->parse_ifconfig();

    # We only really need to look to see if there is an ethernet address
    my @interfaces;
    open(FH, "/bin/ls -1 /sys/class/net |") or die "Could not run /bin/ls: $?\n";
    while(<FH>) {
        chomp;
        if ($_ =~ /^(e.*)/) {		
            if (defined $ifconfigModel->get_interface($1)) {
                push @interfaces, $1;
            }
        }
    }
    close(FH);
    return @interfaces;
}

=item B<interface_get>

    Function   : Gets information about the given interface
    Parameters : sid, interfaceName
    Returns    : hash of interface info with the following keys:
                     NAME=<interface name>
                     STATUS=<ENABLED|DISABLED>
                     MODE=<STATIC|DHCP>
                     ADDRESS=<ip address>
                     MASK=<network mask>
                     GATEWAY=<gateway>
                     BROADCAST=<broadcast>
                     NETWORK=<network>
                 If a session is provided and C<MODE=DHCP>, then C<ADDRESS>, C<MASK>, C<GATEWAY>, C<BROADCAST>, and C<NETWORK> will not be returned.


=cut
sub interface_get {
    my ($sid, $interfaceName) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfaceModelArray = $interfacesModel->get_interface($interfaceName);

    if (! defined @$interfaceModelArray[0]) {
        die "Unknown interface: $interfaceName\n";
    }

    my %interface = (
        NAME => @$interfaceModelArray[0]->name(),
    );

    foreach my $interfaceModel(@$interfaceModelArray) {

        if ($interfaceModel->address_family() eq "inet6"){
            set_if_defined(\%interface, "ADDRESS_IPV6", $interfaceModel->get_option("address"));
            set_if_defined(\%interface, "PREFIXLEN", $interfaceModel->get_option("prefixlength"));
            set_if_defined(\%interface, "GATEWAY_IPV6", $interfaceModel->get_option("gateway"));
            set_if_defined(\%interface, "NETWORK_IPV6", $interfaceModel->get_option("network"));
            
        }else {
            set_if_defined(\%interface, "ADDRESS", $interfaceModel->get_option("address"));
            set_if_defined(\%interface, "MASK", $interfaceModel->get_option("netmask"));
            set_if_defined(\%interface, "GATEWAY", $interfaceModel->get_option("gateway"));
            set_if_defined(\%interface, "NETWORK", $interfaceModel->get_option("network"));
            set_if_defined(\%interface, "BROADCAST", $interfaceModel->get_option("broadcast")); 
            
        }
        if (defined $interfaceModel->config_method()) {
            if($interfaceModel->address_family() eq "inet6"){
                $interface{MODE_IPV6} = uc($interfaceModel->config_method());
            }else{
                $interface{MODE} = uc($interfaceModel->config_method());
            }
        }
    }

    if ($interfacesModel->is_auto($interfaceName)) {
        $interface{STATUS} = "ENABLED";
    } else {
        $interface{STATUS} = "DISABLED";
    }

    if (! defined $sid) {
        # When looking at the live system, make sure we use values from ifconfig / route
        # Note that when dhcp is used, the values are not available from /etc/network/interfaces.
        my $ifconfigModel = Ifconfig->parse_ifconfig();
        my $ifconfigInterfaceModel = $ifconfigModel->get_interface($interfaceName);

        set_if_defined(\%interface, "ADDRESS", $ifconfigInterfaceModel->get_option("address"));
        set_if_defined(\%interface, "MASK", $ifconfigInterfaceModel->get_option("netmask"));
        set_if_defined(\%interface, "BROADCAST", $ifconfigInterfaceModel->get_option("broadcast"));
        set_if_defined(\%interface, "ADDRESS_IPV6", $ifconfigInterfaceModel->get_option("address_ipv6"));
        set_if_defined(\%interface, "PREFIXLEN", $ifconfigInterfaceModel->get_option("prefixlen"));
        set_if_defined(\%interface, "SCOPE", uc($ifconfigInterfaceModel->get_option("scope")));
        set_if_defined(\%interface, "TYPE", $ifconfigInterfaceModel->get_option("type"));
        # Get the gateway address by the route table.
        # Look for the route with the U=up and G=gateway flags for the given interface.

        my $gateway = determine_gateway_for_interface($interfaceName);
        if (defined $gateway) {
            $interface{GATEWAY} = $gateway;
        }
        my $gateway_ipv6 = determine_gateway_for_interface_ipv6($interfaceName);
        if (defined $gateway_ipv6) {
            $interface{GATEWAY_IPV6} = $gateway_ipv6;
        }

    }

    return \%interface;
}

sub determine_gateway_for_interface {
    my ($interfaceName) = @_;

    my @routeModel = parse_route();
    my @candidateRoutes = grep { $_->{FLAGS} =~ /UG/ && $_->{INTERFACE} =~ /$interfaceName/ } @routeModel;
    if (scalar @candidateRoutes >= 1) {
        return $candidateRoutes[0]->{GATEWAY};
    }
    return undef;
}

sub determine_gateway_for_interface_ipv6 {
    my ($interfaceName) = @_;

    my @routeModel = parse_route_ipv6();
    my @candidateRoutes = grep { $_->{FLAGS} =~ /UG/ && $_->{INTERFACE} =~ /$interfaceName/ } @routeModel;
    if (scalar @candidateRoutes >= 1) {
        return $candidateRoutes[0]->{GATEWAY};
    }
    return undef;
}

sub set_if_defined {
    my ($hashRef, $hashKey, $value) = @_;
    if (defined $value) {
        $hashRef->{$hashKey} = $value;
    }
}

=item B<interface_enable>

    Function   : Enables the the given interface (by configuring the 'auto' line in the interfaces file)
    Parameters : sid, interfaceNname
    Throws     : exception on error

=cut
sub interface_enable {
    my ($sid, $interfaceName) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfaceModel = $interfacesModel->get_interface($interfaceName);
    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    if (! $interfacesModel->is_auto($interfaceName)) {
        $interfacesModel->add_auto($interfaceName);
    }

    $interfacesModel->write_interfaces_file($interfacesFile);

    log_info_with_sid($sid, "Enabled interface $interfaceName");
}

=item B<interface_disable>

    Function   : Disables the the given interface (by configuring the 'auto' line in the interfaces file)
    Parameters : sid, interfaceName
    Throws     : exception on error

=cut
sub interface_disable {
    my ($sid, $interfaceName) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfaceModel = $interfacesModel->get_interface($interfaceName);
    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    if ($interfacesModel->is_auto($interfaceName)) {
        $interfacesModel->remove_auto($interfaceName);
    }

    $interfacesModel->write_interfaces_file($interfacesFile);

    log_info_with_sid($sid, "Disabled interface $interfaceName");
}

=item B<interface_set_static>

    Function   : Sets the interface to use the static configuration method (by configuring the 'iface' line in the interfaces file)
    Parameters : sid, interfaceName, hash of (ADDRESS, MASK, GATEWAY)
    Throws     : exception on error

=cut
sub interface_set_static {
    my ($sid, $interfaceName, $interface) = @_;
    my $addressFamily = $interface->{ADDRESSFAMILY};

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfacesModelArray = $interfacesModel->get_interface($interfaceName);

    my $interfaceModel = @$interfacesModelArray[0];

    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    my @newInterfaceModelArray;

    foreach my $interfaceModel(@$interfacesModelArray) {

        my $tempFamily = $interfaceModel->address_family() eq "inet6" ? "IPv6" : "IPv4";
        if($tempFamily eq $addressFamily and ($interfaceModel->config_method() eq "dhcp" or $interfaceModel->config_method() eq "auto")){
        $interfaceModel = setInterfaceModelOptions($addressFamily, $interface, $interfaceModel);
        $interfaceModel->config_method("static");

        push  @newInterfaceModelArray, $interfaceModel;
         } elsif($interfaceModel->config_method() eq "static") {
            my $changeInetFamily = $addressFamily eq "IPv4" ? "inet" : "inet6";
            $interfaceModel = setInterfaceModelOptions($addressFamily, $interface, $interfaceModel);
            $interfaceModel->address_family($changeInetFamily);
            push  @newInterfaceModelArray, $interfaceModel;
         }
    }

    $interfacesModel->{INTERFACES}{$interfaceName} = [@newInterfaceModelArray];

    $interfacesModel->write_interfaces_file($interfacesFile);

    if($addressFamily eq "IPv6"){
        
        enableDisableIPv6($sid,0);

        createNetworkSwitchFile("inet6");

        log_info_with_sid($sid, "set $interfaceName to STATIC mode. address=$interface->{ADDRESS} prefixlength=$interface->{PREFIXLENGTH} gateway=$interface->{GATEWAY}");
    }else {
        enableDisableIPv6($sid,1);
        createNetworkSwitchFile("inet");

        log_info_with_sid($sid, "set $interfaceName to STATIC mode. address=$interface->{ADDRESS} mask=$interface->{MASK} gateway=$interface->{GATEWAY}");
    }

}


sub createNetworkSwitchFile {
  my $networkSwitch = shift;
  open(IPFLAG, ">", "/var/lib/vplatform/network-switch-CONFIG");
  print IPFLAG $networkSwitch;
  close(IPFLAG);
  
  chmod 0766, "/var/lib/vplatform/network-switch-CONFIG" or die "Couldn't change the permission to: $!";
  
}



=item B<enableDisableIPv6>

    Function   : Disable Or Enable IPv6 stack in UA 
    Parameters : sid, 0 - Enable, 1- Disable

=cut



sub enableDisableIPv6 {

    my ($sid, $optn) = @_;

    my $sysctlFile = get_file_to_edit($sid, "SYSCTL");

    my $settingsAlreadyExists;

    open(FILE,"$sysctlFile");
    $settingsAlreadyExists = (grep{/net.ipv6.conf.all.disable_ipv6/} <FILE>);
    close FILE;

    if($settingsAlreadyExists == 1){

        open(FILE, $sysctlFile) || die "File not found";
        my @lines = <FILE>;
        close(FILE);

        my @newlines;
        foreach(@lines) {
            if($optn  == 0){
                $_ =~ s/net.ipv6.conf.all.disable_ipv6 = 1/net.ipv6.conf.all.disable_ipv6 = 0/g;
                $_ =~ s/net.ipv6.conf.default.disable_ipv6 = 1/net.ipv6.conf.default.disable_ipv6 = 0/g;
                $_ =~ s/net.ipv6.conf.lo.disable_ipv6 = 1/net.ipv6.conf.lo.disable_ipv6 = 0/g; 
            } else{
                $_ =~ s/net.ipv6.conf.all.disable_ipv6 = 0/net.ipv6.conf.all.disable_ipv6 = 1/g;
                $_ =~ s/net.ipv6.conf.default.disable_ipv6 = 0/net.ipv6.conf.default.disable_ipv6 = 1/g;
                $_ =~ s/net.ipv6.conf.lo.disable_ipv6 = 0/net.ipv6.conf.lo.disable_ipv6 = 1/g; 
            }
           push(@newlines,$_);
        }


        open(FILE, ">$sysctlFile") || die "File not found";
        print FILE @newlines;
        close(FILE);
    } else{

      open(my $fd, ">>$sysctlFile");
      print $fd "\nnet.ipv6.conf.all.disable_ipv6 = $optn";
      print $fd "\nnet.ipv6.conf.default.disable_ipv6 = $optn";
      print $fd "\nnet.ipv6.conf.lo.disable_ipv6 = $optn";
      close($fd);

    }


}





    # Function   : Set interfacemodel values.
    # Parameters : addressfamily
    # Returns    : new interface model
sub setInterfaceModelOptions {
    my ($addressFamily, $interface, $interfaceModel) = @_;

    if (defined $interface->{ADDRESS}) {
        verify_address($addressFamily,"address", $interface->{ADDRESS});
        $interfaceModel->set_option("address", $interface->{ADDRESS});
    } else {
        die "interface must have ADDRESS\n";
    }

    if($addressFamily eq "IPv6"){

        if (defined $interface->{PREFIXLENGTH}) {
            if ($interface->{PREFIXLENGTH} > 128) {
                 die "PREFIXLENGTH must be less than or equals to 128\n";
            }
            $interfaceModel->set_option("prefixlength", $interface->{PREFIXLENGTH});

            $interfaceModel->set_option("netmask", "");

        } else {
            die "interface must have PREFIXLENGTH\n";
        }

    } else{

        if (defined $interface->{MASK}) {
            verify_address($addressFamily, "mask", $interface->{MASK});
            $interfaceModel->set_option("netmask", $interface->{MASK});
        } else {
            die "interface must have MASK\n";
        }

    }
    if (defined $interface->{GATEWAY}) {
        verify_address($addressFamily,"gateway", $interface->{GATEWAY});
        $interfaceModel->set_option("gateway", $interface->{GATEWAY});
    } else {
        die "interface must have GATEWAY\n";
    }

    if($addressFamily eq "IPv6"){
        $interfaceModel->set_option("network", $interface->{NETWORK});
        $interfaceModel->set_option("broadcast", "");

    }else{
        my ($network, $broadcast) = calculate_network_and_broadcast($interface->{ADDRESS}, $interface->{MASK});
        $interfaceModel->set_option("network", $network);
        $interfaceModel->set_option("broadcast", $broadcast);

        }

        return $interfaceModel;
}

=item B<calculate_network_and_broadcast>

    Function   : Calculates the IPv4 network and broadcast addresses from the address and mask.
    Parameters : address, mask
    Returns    : network, broadcast

=cut
sub calculate_network_and_broadcast {
    my ($address, $mask) = @_;

    # Calculate the network and broadcast values

    # Split the IP address and net mask addresses by . into 4 string segments
    my @ipSegments = split /\./, $address;
    my @netMaskSegments = split /\./, $mask;

    my $network;
    my $broadcast;
    for (my $i = 0; $i < 4; $i++) {
        # Add zero to the strings to force conversion to a number
        my $ipSegment = $ipSegments[$i] + 0;
        my $netMaskSegment = $netMaskSegments[$i] + 0;

        # Bitwise AND the segments together to produce the network segment
        my $netSegment = $ipSegment & $netMaskSegment;

        # Bitwise OR the address segment with the inverse of the mask segment to produce the broadcast segment
        my $broadcastSegment = $ipSegment | ($netMaskSegment ^ 0xFF);

        if ($i == 0) {
            $network = $netSegment;
            $broadcast = $broadcastSegment;
        } else {
            $network = $network . '.' . $netSegment;
            $broadcast = $broadcast . '.' . $broadcastSegment;
        }
    }

    return ($network, $broadcast);
}

=item B<interface_set_dhcp>

    Function   : Sets the interface to use the dhcp configuration method (by configuring the 'iface' line in the interfaces file)
    Parameters : sid, interfaceName
    Throws     : exception on error

=cut
sub interface_set_dhcp {
    my ($sid, $interfaceName) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfacesModelArray = $interfacesModel->get_interface($interfaceName);

    my $interfaceModel = @$interfacesModelArray[0];

    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    my $config = ($interfaceModel->address_family() eq "inet6" ? "auto" : "dhcp");

    $interfaceModel->config_method($config);
    $interfaceModel->set_option("address", undef);
    $interfaceModel->set_option("netmask", undef);
    $interfaceModel->set_option("prefixlength", undef);
    $interfaceModel->set_option("gateway", undef);
    $interfaceModel->set_option("network", undef);
    $interfaceModel->set_option("broadcast", undef);
    $interfaceModel->set_option("dns-domain", undef);
    $interfaceModel->set_option("dns-search", undef);
    $interfaceModel->set_option("dns-nameservers", undef);

    my @temparr = $interfacesModel->{INTERFACES}{$interfaceName}[0];

    my $interfaceModelTemp = Interface->new();
    $interfaceModelTemp->name($interfaceName);
    $interfaceModelTemp->address_family($interfaceModel->address_family() eq "inet6" ? "inet" : "inet6");
    $interfaceModelTemp->config_method($interfaceModelTemp->address_family() eq "inet" ? "dhcp" : "auto");

    push @temparr, $interfaceModelTemp;

    $interfacesModel->{INTERFACES}{$interfaceName}=  [@temparr];

    # print Dumper($interfacesModel);

    $interfacesModel->write_interfaces_file($interfacesFile);

    log_info_with_sid($sid, "set $interfaceName to DHCP/AUTO mode.");

    enableDisableIPv6($sid,0);

    createNetworkSwitchFile("dhcp");
}

=item B<dns_get>

    Function   : Gets information about dns configuration
    Parameters : sid
    Returns    : hash of dns info with the following keys:
                     NAMESERVERS=<nameserver1,nameserver2,...>
                     DOMAIN=<domain name>
                     SEARCHDOMAINS=<searchdomain1,searchdomain2,...>

=cut
sub dns_get {
    my ($sid) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my %dns;

    foreach my $interfacesModelArray (values %{$interfacesModel->get_interfaces()}) {

        my $interfaceModel = @$interfacesModelArray[0];

        set_if_defined(\%dns, "DOMAIN", $interfaceModel->get_option("dns-domain"));

        if (defined $interfaceModel->get_option("dns-nameservers")) {
            my @nameservers = split(/\s+/, $interfaceModel->get_option("dns-nameservers"));
            push_uniq(\@{$dns{NAMESERVERS}}, @nameservers);
        }

        if (defined $interfaceModel->get_option("dns-search")) {
            my @searchDomains = split(/\s+/, $interfaceModel->get_option("dns-search"));
            push_uniq(\@{$dns{SEARCHDOMAINS}}, @searchDomains);
        }
    }

    if (! defined $sid) {
        # When looking at the live system, make sure we use values from /etc/resolv.conf
        my $resolvConfModel = parse_resolve_conf();

        if (defined $resolvConfModel->{"domain"}) {
            $dns{DOMAIN} = @{$resolvConfModel->{"domain"}}[0];
        }
        if (defined $resolvConfModel->{"nameserver"}) {
            push_uniq(\@{$dns{NAMESERVERS}}, @{$resolvConfModel->{"nameserver"}});
        }
        if (defined $resolvConfModel->{"search"}) {
            push_uniq(\@{$dns{SEARCHDOMAINS}}, @{$resolvConfModel->{"search"}});
        }
    }

    return \%dns;
}

=item B<dns_set>

    Function   : sets information about dns configuration
    Parameters : sid,  hash of (NAMESERVERS, DOMAIN, SEARCHDOMAINS) as returned by dns_get

=cut
sub dns_set {
    my ($sid, $addressfamily, $dns) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    if (defined $dns->{DOMAIN}) {
        verify_fqdn("domain name", $dns->{DOMAIN});
    }
    if (defined $dns->{NAMESERVERS}) {
        foreach my $nameserver (@{$dns->{NAMESERVERS}}) {
		#TODO uncomment this when IPv6 Dns will be availbe
            verify_address($addressfamily,"nameserver", $nameserver);
        }
    }
    if (defined $dns->{SEARCHDOMAINS}) {
        foreach my $searchDomain (@{$dns->{SEARCHDOMAINS}}) {
            verify_fqdn("search domain", $searchDomain);
        }
    }

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    foreach my $interfacesModelArray (values %{$interfacesModel->get_interfaces()}) {

        my $interfaceModel = @$interfacesModelArray[0];


        if (! ($interfaceModel->name() =~ /^eth\d/)) {
            # only set dns info on eth* interfaces
            next;
        }

        if (defined $dns->{NAMESERVERS}) {
            $interfaceModel->set_option("dns-nameservers", join(" ", @{$dns->{NAMESERVERS}}));
        }
        if (defined $dns->{DOMAIN}) {
            $interfaceModel->set_option("dns-domain", $dns->{DOMAIN});
        }
        if (defined $dns->{SEARCHDOMAINS}) {
            $interfaceModel->set_option("dns-search", join(" ", @{$dns->{SEARCHDOMAINS}}));
        }
    }

    if (defined $dns->{DOMAIN} && $dns->{DOMAIN} ne "") {
        update_hosts_file($sid, hostname_get($sid), $dns->{DOMAIN});
    }

    $interfacesModel->write_interfaces_file($interfacesFile);

    if (defined $dns->{NAMESERVERS}) {
        log_info_with_sid($sid, "set dns nameservers to " . join(" ", @{$dns->{NAMESERVERS}}));
    }
    if (defined $dns->{DOMAIN}) {
        log_info_with_sid($sid, "set dns domain to $dns->{DOMAIN}");
    }
    if (defined $dns->{SEARCHDOMAINS}) {
        log_info_with_sid($sid, "set dns search domains to " . join(" ", @{$dns->{SEARCHDOMAINS}}));
    }
}



=item B<route_list>

    Function   : Returns a list of routes.
    Parameters : sid, interface name
    Returns    : List of references to hashes of route info with the following keys:
                     TYPE=<-net|-host>
                     DESTINATION=<destination>
                     GATEWAY=<gateway>

=cut
sub route_list {
    my ($sid, $interfaceName) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfacesModelArray = $interfacesModel->get_interface($interfaceName);
   
   my $interfaceModel = @$interfacesModelArray[0];

    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    return map {
            {
                TYPE => $_->type(),
                DESTINATION => $_->destination(),
                GATEWAY => $_->gateway(),
            }
        } @{$interfaceModel->routes()};
}


=item B<route_add>

    Function   : Adds the given route to the given interface
    Parameters : sid, interfaceName, hash of (DESTINATION, GATEWAY)
    Throws     : exception on error

=cut
sub route_add {
    my ($sid, $interfaceName, $route) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfacesModelArray = $interfacesModel->get_interface($interfaceName);
   
    my $interfaceModel = @$interfacesModelArray[0];

    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    my $routeModel = Route->new();
    $routeModel->type(determine_destination_type($route->{DESTINATION}));
    $routeModel->destination($route->{DESTINATION});
    $routeModel->gateway($route->{GATEWAY});
    $interfaceModel->add_route($routeModel);

    $interfacesModel->write_interfaces_file($interfacesFile);

    log_info_with_sid($sid, "add route to $interfaceName. destination=$route->{DESTINATION} gateway=$route->{GATEWAY}");
}

=item B<route_delete>

    Function   : Deletes the given route from the given interface
    Parameters : sid, interfaceName, hash of (DESTINATION, GATEWAY)
    Throws     : exception on error

=cut
sub route_delete {
    my ($sid, $interfaceName, $route) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfacesModelArray = $interfacesModel->get_interface($interfaceName);
   
    my $interfaceModel = @$interfacesModelArray[0];

    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    $interfaceModel->remove_route($route);

    $interfacesModel->write_interfaces_file($interfacesFile);

    log_info_with_sid($sid, "delete route from $interfaceName. destination=$route->{DESTINATION} gateway=$route->{GATEWAY}");
}

=item B<routes_set>

    Function   : Sets all routes at once for a given interface.  Completely replaces existing route definitions.
    Parameters : sid, interfaceName, hashes of (DESTINATION, GATEWAY)
    Throws     : exception on error

=cut
sub routes_set {
    my ($sid, $interfaceName, @routes) = @_;

    my $interfacesFile = get_file_to_edit($sid, "INTERFACES");

    my $interfacesModel = Interfaces->parse_interfaces_file($interfacesFile);

    my $interfaceModel = $interfacesModel->get_interface($interfaceName);
    if (! defined $interfaceModel) {
        die "Unknown interface: $interfaceName\n";
    }

    log_info_with_sid($sid, "delete all routes from $interfaceName");
    $interfaceModel->remove_all_routes();

    foreach my $route (@routes) {
        my $routeModel = Route->new();
        $routeModel->type(determine_destination_type($route->{DESTINATION}));
        $routeModel->destination($route->{DESTINATION});
        $routeModel->gateway($route->{GATEWAY});
        $interfaceModel->add_route($routeModel);
       log_info_with_sid($sid, "add route to $interfaceName. destination=$route->{DESTINATION} gateway=$route->{GATEWAY}");
    }

    $interfacesModel->write_interfaces_file($interfacesFile);
}

=item B<hostname_get>

    Function   : Gets the hostname (w/o the domain name)
    Parameters : sid (optional)
    Returns    : If sid is provided, return the hostname from the edit session.
                 Else the hostname of the live system.

=cut
sub hostname_get {
    my ($sid) = @_;

    my $hostname;
    if (defined $sid) {
        my $hostnameFile = get_file_to_edit($sid, "HOSTNAME");
        $hostname = parse_hostname_file($hostnameFile);
    } else {
        # When looking at the live system, make sure we use values from `hostname`
        $hostname = `hostname`;
        chomp $hostname;
    }

    return $hostname;
}

=item B<hostname_set>

    Function   : Sets the hostname
    Parameters : sid, hostname

=cut
sub hostname_set {
    my ($sid, $hostname) = @_;

    # Update the hostname file
    my $hostnameFile = get_file_to_edit($sid, "HOSTNAME");

    verify_hostname($hostname);

    local *FH;
    open(FH, ">$hostnameFile") or die "Could not open file $hostnameFile for writing: $!";
    print FH "$hostname\n";
    close(FH);

    my $dns = dns_get($sid);
    update_hosts_file($sid, $hostname, $dns->{DOMAIN});

    log_info_with_sid($sid, "set hostname to $hostname");
}

=item B<update_hosts_file>

    Function   : Updates the host file with the hostname and domain name (if given)
    Parameters : sid, hostname, domain name (optional)

=cut
sub update_hosts_file {
    my ($sid, $hostname, $domainname) = @_;

    my $hostsFile = get_file_to_edit($sid, "HOSTS");
    open(FH, $hostsFile) or die "Failed to read from $hostsFile: $!";
    my @hostLines = <FH>;
    close(FH);

    open(FH, ">$hostsFile") or die "Failed to open $hostsFile for writing: $!";
    # Add localhost info
    print FH "127.0.0.1\tlocalhost\n";
    if ($domainname ne "") {
        # include the fqdn here so that the dns domain name is accurately reported by dnsdomainname and java
        print FH "127.0.1.1\t$hostname.$domainname $hostname\n";
    } else {
        print FH "127.0.1.1\t$hostname\n";
    }

    # Add any lines from the original hosts file that don't refer to the localhost (127.0.0.1) or the host name (127.0.1.1)
    foreach my $hostLine (@hostLines) {
        if (index($hostLine, "127.0.") < 0) {
            print FH $hostLine;
        }
    }
    close(FH);
}

=item B<config_get>

    Function   : Gets the entire networking configuration as a hash reference.  See B<network-config config get> for hash contents.
    Parameters : sid

=cut
sub config_get {
    my ($sid) = @_;

    my %config;

    $config{HOSTNAME} = hostname_get($sid);

    # This is not very efficient, because we end up parsing multiple times.
    # TODO Need to refactor this to limit the amount of parsing required.

    foreach my $interfaceName ( interface_list() ) {
        my $interface = interface_get($sid, $interfaceName);

        push @{$config{"INTERFACES"}}, $interface->{NAME};

        while (my ($key, $value) = each %$interface) {
            $config{"INTERFACE_$interface->{NAME}_$key"} = $value;
        }

        foreach my $route ( route_list($sid, $interfaceName) ) {
            push @{$config{"INTERFACE_$interface->{NAME}_ROUTES"}}, "$route->{DESTINATION} $route->{GATEWAY}"; #"
        }
    }

    my $dns = dns_get($sid);
    while (my ($key, $value) = each %$dns) {
        $config{$key} = $value;
    }

    return \%config;

}

=item B<config_set>

    Function   : Sets the entire networking configuration.  See B<network-config config get> for hash contents.
    Parameters : config as a hash ref, skipRefresh (true if the live system should NOT be refreshed after the persistent files are updated.  default is false)

=cut
sub config_set {
    my ($config, $skipRefresh) = @_;

    my $sid = edit_begin();

    eval {
        if (defined $config->{"HOSTNAME"} && defined $config->{"DOMAIN"}) {
            verify_fqdn("FQDN", $config->{"HOSTNAME"} . "." . $config->{"DOMAIN"});
        }

        if (defined $config->{"HOSTNAME"}) {
            hostname_set($sid, $config->{"HOSTNAME"});
        }

        # This is not very efficient, because we end up parsing multiple times.
        # TODO Need to refactor this to limit the amount of parsing required.


        if (defined $config->{"NAMESERVERS"}
                || defined $config->{"DOMAIN"}
                || defined $config->{"SEARCHDOMAINS"}) {

            # This works because dns_set expects the same three keys
            dns_set($sid, $config);
        }

        foreach my $interfaceName ( interface_list() ) {

            # interface status (ENABLED/DISABLED)
            if (defined $config->{"INTERFACE_${interfaceName}_STATUS"}) {

                if ($config->{"INTERFACE_${interfaceName}_STATUS"} eq "ENABLED") {
                    interface_enable($sid, $interfaceName);
                } elsif ($config->{"INTERFACE_${interfaceName}_STATUS"} eq "DISABLED") {
                    interface_disable($sid, $interfaceName);
                } else {
                    die "Unknown interface status: " . $config->{"INTERFACE_${interfaceName}_STATUS"} . "\n";
                }
            }

            # interface mode (STATIC/DHCP)
            if (defined $config->{"INTERFACE_${interfaceName}_MODE"}) {
                if (uc($config->{"INTERFACE_${interfaceName}_MODE"}) eq "STATIC") {
                    interface_set_static($sid, $interfaceName, {
                        ADDRESS => $config->{"INTERFACE_${interfaceName}_ADDRESS"},
                        MASK => $config->{"INTERFACE_${interfaceName}_MASK"},
                        GATEWAY => $config->{"INTERFACE_${interfaceName}_GATEWAY"},
                    });
                } elsif (uc($config->{"INTERFACE_${interfaceName}_MODE"}) eq "DHCP") {
                    interface_set_dhcp($sid, $interfaceName);
                } else {
                    die "Unknown interface mode: " . $config->{"INTERFACE_${interfaceName}_MODE"} . "\n";
                }
            }


            # routes
            if (defined $config->{"INTERFACE_${interfaceName}_ROUTES"}) {

                my @routes = map {
                        my ($destination, $gateway) = split /\s+/, $_;
                        return {
                            DESTINATION => $destination,
                            GATEWAY => $gateway
                        }
                    } @{$config->{"INTERFACE_${interfaceName}_ROUTES"}};

                routes_set($sid, $interfaceName, @routes);
            }
        }

        edit_commit($sid, $skipRefresh);

        # Needed so that the exception handling block does not get executed
        1;
    } or do {
        my $error = $@;
        if (sid_exists($sid)) {
            edit_rollback($sid);
        }
        die $error;
    }
}

=item B<config_validate>

    Function   : Tests the current live configuration to determine if it is valid (by pinging the gateway).
    Parameters : interface name (optional, defaults to eth0)
    Returns    : undef if valid.  Error string if invalid

=cut
sub config_validate {

    my ($interfaceName) = @_;

    if (! defined $interfaceName) {
	# Find the first ethernet interface
        open(FH, "/bin/ls -1 /sys/class/net |") or die "Could not run /bin/ls: $?\n";
        while(<FH>) {
            chomp;
            if ($_ =~ /^(e.*)/) {		
                $interfaceName = $1;
                last;
            }
        }
        close(FH);
    }

    my $ifconfigModel = Ifconfig->parse_ifconfig();

    my $interface = $ifconfigModel->get_interface($interfaceName);

    if (! defined $interface) {
        #return "Unable to find interface: $interfaceName";
	    return -1;
    }

    my $address = $interface->get_option("address");
    if (! defined $address) {
        #return "Unable to find address for interface: $interfaceName";
	    return -2;
    }

    my $gateway = determine_gateway_for_interface($interfaceName);
    if (! defined $gateway) {
        #return "Unable to find gateway for interface: $interfaceName";
	    return -3;
    }

    my $pingSuccess = 0;
    open(PING, "ping -c 1 $gateway |") or die "Failed to execute ping";
    while(<PING>) {
        # Look for the ping success
        if ($_ =~ /1 received/) {
            $pingSuccess = 1;
        }
    }
    close(PING);

    if ($pingSuccess == 0) {
        #return "Cannot ping gateway: $gateway";
	    return -4;
    }
    return undef;
}

sub verify_address {
    my ($addressfamily, $fieldName, $address) = @_;

     if($addressfamily eq "IPv6"){

        my $IP6_PATTERN = qr/^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$/;        

        if ($address =~ /$IP6_PATTERN/) {
        } else {
             die ("Invalid $fieldName: $address\n");
        }
    } else {
        if ($address =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)$/) {

        if ($1 > 255 || $2 > 255 || $3 > 255 || $4 > 255) {
            die "Invalid $fieldName: $address.  All segments must be less than or equal to 255\n";
        }

        } else {
            die "Invalid $fieldName: $address\n";
        }
    }
}

sub verify_hostname {
    my ($hostname) = @_;
    if (length($hostname) > 64) {
        die "Hostname must be less than or equal to 64 characters.\n";
    }
    if ($hostname =~ "[^a-zA-Z0-9\-]") {
        die "Hostname contains invalid characters. Only letters a-z (case insensitive), digits 0-9, and the hyphen (-) are allowed.\n";
    }
    if ($hostname =~ "^-") {
        die "Hostname must not start with a hyphen.\n";
    }
}

sub verify_fqdn {
    my ($fieldName, $fqdn) = @_;

    if (length($fqdn) > 255) {
        die "$fieldName must be less than or equal to 255 characters.\n";
    }
    if ($fqdn =~ "[^a-zA-Z0-9\-.]") {
        die "$fieldName contains invalid characters. Only letters a-z (case insensitive), digits 0-9, the hyphen (-), and dot (.) are allowed.\n";
    }

    foreach my $segment (split(/\./, $fqdn)) {
        if ($segment =~ "^-") {
            die "$fieldName must not contain segments that start with a hyphen.\n";
        }
        if ($segment eq "") {
            die "$fieldName must not contain empty segments.\n";
        }
        if (length($segment) > 64) {
            die "All segments in $fieldName must be less than 64 characters.\n";
        }
    }
}


sub parse_hostname_file {
    my ($hostnameFile) = @_;
    local *FH;
    open(FH, "<$hostnameFile") or die "Could not open file $hostnameFile for reading: $!\n";

    my $hostname;
    while (<FH>) {
        chomp;

        if ($_ =~ /\s*#/) {
            # skip comment lines
            next;
        }

        if ($_ =~ /\s+/) {
            # skip blank lines
            next;
        }

        $hostname = $_;
    }

    close(FH);
    return $hostname;
}


sub determine_destination_type {
    my ($destination) = @_;

    # Destination is a network, if it contains a /  (e.g. "192.168.45.0/16") or it ends with ".0" (e.g. "192.168.45.0")

    if ($destination =~ /\//) {
        return "-net";
    }

    if ($destination =~ /\.0$/) {
        return "-net";
    }

    # Otherwise, assume it's a host

    return "-host";
}

sub parse_resolve_conf {
    my %resolveConfModel;

    local *FH;
    open(FH, "</etc/resolv.conf") or die "Could not open /etc/resolv.conf for reading: $!\n";
    while (<FH>) {
        chomp;

        # ignore comments
        if ($_ =~ /^\s*#/) {
            next;
        }

        if ($_ =~ /^\s*(\S+)\s+(.*)\s*$/) {
            my @values = split(/\s+/, $2);
            push_uniq(\@{$resolveConfModel{$1}}, @values);
        }
    }

    close(FH);
    return \%resolveConfModel;
}



=item B<push_uniq>

    Function   : pushes elements onto an array if and only if those elements don't already exist in the array
    Parameters : array reference, elements to add

=cut
sub push_uniq {
    my ($arrayRef, @toAdd) = @_;

    if (ref($arrayRef) ne 'ARRAY') {
        die "Expected array reference as first argument";
    }

    foreach my $element (@toAdd) {
        if (ref($element) ne '') {
            die "Expected array as second argument.";
        }

        if (!grep(/^$element$/, @{${arrayRef}})) {
            push(@{$arrayRef}, $element);
        }
    }
}

sub parse_route {
    my @routeModel;

    # This is what we're parsing:
    #
    # Kernel IP routing table
    # Destination     Gateway         Genmask         Flags Metric Ref    Use Iface
    # 10.225.150.0    0.0.0.0         255.255.255.0   U     0      0        0 eth0
    # 0.0.0.0         10.225.150.1    0.0.0.0         UG    0      0        0 eth0


    local *FH;
    open(FH, "route -n |") or die "Could not run route: $!\n";
    # skip the first two lines
    <FH>;
    <FH>;
    while (<FH>) {
        chomp;

        my @line = split(/\s+/, $_);

        my %route = (
            DESTINATION => $line[0],
            GATEWAY => $line[1],
            GENMASK => $line[2],
            FLAGS => $line[3],
            INTERFACE => $line[7],
        );
        push @routeModel, \%route;

    }

    close(FH);
    return @routeModel;
}

sub parse_route_ipv6 {
    my @routeModel;

    # This is what we're parsing:
    #
    # Kernel IP routing table
    #Destination                    Next Hop                   Flag Met Ref Use If
	#fd20:8b1e:b255:8567::/64       [::]                       UAe  256 0     0 eth0
	#fe80::/64                      [::]                       U    256 0     0 eth0
	#[::]/0                         fe80::5:73ff:fea0:116      UGDAe 1024 0     0 eth0
	#[::]/0                         [::]                       !n   -1  1     1 lo
	#localhost/128                  [::]                       Un   0   3    22 lo
	#fd20:8b1e:b255:8567:250:56ff:fe8f:ebe7/128 [::]                       Un   0   1     0 lo
	#fe80::250:56ff:fe8f:ebe7/128   [::]                       Un   0   3    13 lo
	#ip6-mcastprefix/8              [::]                       U    256 1    26 eth0
	#[::]/0                         [::]                       !n   -1  1     1 lo



    local *FH;
    open(FH, "route -6 |") or die "Could not run route: $!\n";
    # skip the first two lines
    <FH>;
    <FH>;
    while (<FH>) {
        chomp;

        my @line = split(/\s+/, $_);

        my %route = (
            DESTINATION => $line[0],
            GATEWAY => $line[1],
            FLAGS => $line[2],
            INTERFACE => $line[6],
        );
        push @routeModel, \%route;

    }

    close(FH);
    return @routeModel;
}



=item B<delete_sid>

    Function   : Deletes temporary files for an edit session
    Parameters : sid
    Throws     : exception if cannot delete files

=cut
sub delete_sid {
    my ($sid) = @_;
    verify_sid($sid);

    my ($backupDir, $editDir) = get_session_dirs($sid);

    foreach my $fileId ( keys %files ) {
        unlink "$backupDir/$files{$fileId}" or die "Cannot create temp files for edit session: $!";
        unlink "$editDir/$files{$fileId}" or die "Cannot create temp files for edit session: $!";
    }

    rmdir $backupDir;
    rmdir $editDir;

    rmdir $sid or warn "Unable to remove temp files for edit session: $!";
}

=item B<verify_sid>

    Function   : Verifies that a sid exists
    Parameters : sid
    Throws     : exception if sid does not exist

=cut
sub verify_sid {
    my ($sid) = @_;
    if (! sid_exists($sid)) {
        die "Invalid session id: $sid\n";
    }
}

=item B<sid_exists>

    Function   : Return 1 if sid exists, 0 otherwise
    Parameters : sid

=cut
sub sid_exists {
    my ($sid) = @_;
    return -d "$sid";
}

=item B<get_session_dirs>

    Function   : Returns the backup and edit dirs for the session with the given sid
    Parameters : sid
    Returns    : backup dir and edit dir

=cut
sub get_session_dirs {
    my ($sid) = @_;
    verify_sid($sid);
    my $backupDir = "$sid/$backupSubdir";
    my $editDir = "$sid/$editSubdir";
    return $backupDir, $editDir;
}

=item B<get_file_to_edit>

    Function   : Returns the file with the given id to edit, based on if a sid is defined
    Parameters : sid, fileId (INTERFACES, etc)
    Returns    : If sid is defined, then returns the file with the file id from the edit dir
                 If sid is not defined, then returns the file with the file id from the live filesystem

=cut
sub get_file_to_edit {
    my ($sid, $fileId) = @_;

    if (defined $sid) {
        verify_sid($sid);
        my ($backupDir, $editDir) = get_session_dirs($sid);
        return "$editDir/$files{$fileId}";
    } else {
        return "$liveDirs{$fileId}/$files{$fileId}";
    }
}

sub log_warn_with_sid {
    my ($sid, $message, @args) = @_;
    printf STDERR "$message\n", @args;
    log_warn("Edit session: $sid - $message", @args);
}

sub log_info_with_sid {
    my ($sid, $message, @args) = @_;
    log_info("Edit session: $sid - $message", @args);
}

=item B<log_info, log_debug, log_warn>

    Function   : Logs message at the appropriate level
    Parameters : message, arguments

=cut
sub log_info {
    my ($message, @args) = @_;
    logit("LOG_INFO", $message, @args);
}
sub log_debug {
    my ($message, @args) = @_;
    logit("LOG_DEBUG", $message, @args);
}
sub log_warn {
    my ($message, @args) = @_;
    logit("LOG_WARNING", $message, @args);
}
sub logit {
    my ($priority, $message, @args) = @_;
    syslog($priority, $message, @args);
}

# Add a callback to log the reason why we died
$SIG{__DIE__} = sub {
    log_warn("Died: ". join(" ", @_));
};

BEGIN {
    openlog("network-config", 'nofatal', "LOG_USER");
}
END {
    closelog;
}

=back

=cut

1;
