#!/usr/bin/perl

# $Id:  $
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.

package Interface;

use strict;

=head1 NAME

Interface - Model of a single interface stanza in the /etc/network/interface file.

=head1 DESCRIPTION

Model of a single interface stanza in the /etc/network/interface file.

For example:

iface eth0 inet static
        address 10.225.150.97
        netmask 255.255.255.0
        network 10.225.150.0
        broadcast 10.225.150.255
        gateway 10.225.150.1
        dns-nameservers 10.60.132.40
        dns-domain gdl.englab.netapp.com
        dns-search gdl.englab.netapp.com hq.netapp.com
        up route add -net 10.0.0.0/16 gw 10.0.0.1 dev eth0
        down route del -net 10.0.0.0/16 gw 10.0.0.1 dev eth0


=head1 SUBROUTINES

=over 4

=item B<new>

    Function   : construct a new Interface object.
    Returns    : A new empty Interface object.

=cut
sub new {
    my $self = {};
    $self->{NAME} = undef;
    $self->{ADDRESS_FAMILY} = undef,
    $self->{CONFIG_METHOD} = undef,
    $self->{OPTIONS} = {};
    $self->{ROUTES} = [];
    bless($self);
    return $self;
}

=item B<name>

    Function   : Get/set the name of the interface
    Parameters : new name

=cut
sub name {
    my $self = shift;
    if (@_) { $self->{NAME} = shift }
    return $self->{NAME};
}

=item B<address_family>

    Function   : Get/set the address family (inet, inet6, etc)
    Parameters : new address family

=cut
sub address_family {
    my $self = shift;
    if (@_) { $self->{ADDRESS_FAMILY} = shift }
    return $self->{ADDRESS_FAMILY};
}

=item B<config_method>

    Function   : Get/set the config method (loopback, static, dhcp. etc)
    Parameters : new config method

=cut
sub config_method {
    my $self = shift;
    if (@_) { $self->{CONFIG_METHOD} = shift }
    return $self->{CONFIG_METHOD};
}

=item B<options>

    Function   : Get a reference to a hash of option name / values

=cut
sub options {
    my $self = shift;
    return $self->{OPTIONS};
}

=item B<set_option>

    Function   : Sets an option value
    Parameters : option name, option value

=cut
sub set_option {
    my $self = shift;
    my ($optionName, $optionValue) = @_;

    if (defined $optionValue) {
        $self->{OPTIONS}{$optionName} = $optionValue;
    } else {
        delete $self->{OPTIONS}{$optionName};
    }
}

=item B<get_option>

    Function   : Gets an option value
    Parameters : option name
    Returns    : option value

=cut
sub get_option {
    my $self = shift;
    my ($optionName) = @_;
    return $self->{OPTIONS}{$optionName};
}

=item B<routes>

    Function   : Get a reference to the list of Routes defined on this Interface

=cut
sub routes {
    my $self = shift;
    return $self->{ROUTES};
}

=item B<add_route>

    Function   : Adds a Route to this interface
    Parameters : Route

=cut
sub add_route {
    my $self = shift;
    my ($route) = @_;
    push @{$self->{ROUTES}}, $route;
}

=item B<remove_route>

    Function   : Removes a Route from this interface
    Parameters : reference to hash containing DESTINATION and GATEWAY keys

=cut
sub remove_route {
    my $self = shift;
    my ($route) = @_;

    # remove the route from the list
    @{$self->{ROUTES}} =
        grep
            { $_->destination() ne $route->{DESTINATION} || $_->gateway() ne $route->{GATEWAY} }
            @{$self->{ROUTES}};
}

=item B<remove_all_routes>

    Function   : Removes all routes from this interface

=cut
sub remove_all_routes {
    my $self = shift;

    @{$self->{ROUTES}} = ();
}

=back

=cut

1; # so the require or use succeeds

