#!/usr/bin/perl

# $Id:  $
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.

package Interfaces;

use strict;
use NetConfig::Interface;
use NetConfig::Route;
use Data::Dumper;

=head1 NAME

Interfaces - Model of the /etc/network/interfaces file.

=head1 DESCRIPTION

The Interfaces class models the contents of the /etc/network/interfaces file.

It is meant to be read from a file, then modified, then written back out to the file.
This simplifies performing modifications of the file.

Use C<Interfaces-E<gt>parse_interfaces_file> to construct a new Interfaces
object from an existing file.

Use C<$interfaces-E<gt>write_interfaces_file> to write the current model
to a file.

=head1 SUBROUTINES

=over 4

=item B<new>

    Function   : construct a new Interfaces object.  Generally you should use parse_interfaces_file instead.
    Returns    : A new empty Interfaces object.

=cut
sub new {
    my $self = {};
    $self->{AUTO} = ();
    $self->{INTERFACES} = {};
    bless($self);
    return $self;
}

=item B<add_auto>

    Function   : Adds interfaces to the auto line
    Parameters : List of interface names to add to the 'auto' line

=cut
sub add_auto {
    my $self = shift;
    push @{$self->{AUTO}}, @_;
}

=item B<remove_auto>

    Function   : Removes an interface from the auto line
    Parameters : interface name to remove from the auto line

=cut
sub remove_auto {
    my $self = shift;
    my ($interfaceName) = @_;
    @{$self->{AUTO}} = grep { $_ ne $interfaceName } @{$self->{AUTO}};
}

=item B<is_auto>

    Function   : Returns true if the given interface is on the auto line
    Parameters : interface name to check to see if is on the auto line

=cut
sub is_auto {
    my $self = shift;
    my ($interfaceName) = @_;

    return scalar grep(/$interfaceName/, @{$self->{AUTO}});
}

=item B<add_interface>

    Function   : Adds an Interface to the Interfaces model
    Parameters : Interface to add

=cut
sub add_interface {
    my $self = shift;
    my ($interface) = @_;
    if (exists($self->{INTERFACES}{$interface->name()})) 
    {

        my @arr = $self->{INTERFACES}{$interface->name()}[0];

        push @arr, $interface;

        $self->{INTERFACES}{$interface->name()}=  [@arr] ;
        
       
    } 
    else 
    {

      my @arr = $interface;
      $self->{INTERFACES}{$interface->name()} = [@arr];

    }
}

=item B<get_interface>

    Function   : Gets the Interface with the given name
    Parameters : interface name to retrieve
    Returns    : Interface

=cut
sub get_interface {
    my $self = shift;
    my ($interfaceName) = @_;
    return $self->{INTERFACES}{$interfaceName};
}

=item B<get_interfaces>

    Function   : Gets all the Interfaces
    Returns    : reference to Hash of interfaceName to Interface objects

=cut
sub get_interfaces {
    my $self = shift;
    return $self->{INTERFACES};
}

=item B<parse_interfaces_file>

    Function   : Parses the given interfaces file into an Interfaces object
    Parameters : path to interfaces file to parse
    Returns    : Interfaces object representation of the interfaces file

=cut
sub parse_interfaces_file {
    my $self = shift;
    my ($interfacesFile) = @_;

    # We're parsing something that looks similar to this:
    # (See also `man interfaces`)
    #
    # For IPv4 Static
    # auto lo eth0
    #
    # iface eth0 inet static
    #         address 10.225.150.97
    #         broadcast 10.225.150.255
    #         dns-domain gdl.englab.netapp.com
    #         dns-nameservers 10.60.132.40
    #         dns-search gdl.englab.netapp.com hq.netapp.com
    #         gateway 10.225.150.1
    #         netmask 255.255.255.0
    #         network 10.225.150.0
    #         up route add -host 10.10.10.50 gw 10.10.10.1 dev eth0
    #         down route del -host 10.10.10.50 gw 10.10.10.1 dev eth0
    #
    # For IPv6 static configuration
    #
    # iface eth1 inet dhcp
    #
    # iface lo inet loopback
    # auto lo eth0
    #
    # iface eth0 inet6 static
    #         address fd20:8b1e:b255:8567:250:56ff:fe8f:b7f0/64
    #         gateway fe80::5:73ff:fea0:116
    #
    # iface eth1 inet dhcp
    #
    # iface lo inet6 loopback

    my $interfaces = new();

    # true when we're inside an interface stanza
    # (which starts with iface, and ends with a non-comment line that starts with non-whitespace)
    my $inStanza = 0;

    # The current interface being constructed (while in an interface stanza)
    my $currentInterface = undef;

    local *FH;
    open(FH, "<$interfacesFile") or die "Could not open file $interfacesFile for reading: $!\n";
    while (<FH>) {
        chomp;

        if ($_ =~ /\s*#/) {
            # skip comment lines
            next;
        }
        if ($_ =~ /^auto\s+(.*)$/) {
            my @autoInterfaces = split(/\s+/, $1);
            # can potentially be more that one auto line (although, we only write out one)
            $interfaces->add_auto(@autoInterfaces);
        }
        if ($_ =~ /^iface/ || $_ =~ /^mapping/ || $_ =~ /^auto/ || $_ =~ /^allow-/) {
            # When we hit a line that starts with a keyword, exit the existing stanza.
            $inStanza = 0;
        }

        if (!$inStanza) {
            if ($_ =~ /^iface\s+(.*)\s+(.*)\s+(.*)/) {
                $inStanza = 1;
                my $interfaceName = $1;
                $currentInterface = Interface->new();
                $currentInterface->name($interfaceName);
                $currentInterface->address_family($2);
                $currentInterface->config_method($3);
                $interfaces->add_interface($currentInterface);
            }
        } else {
            my $currentInterfaceName = $currentInterface->name();
            if ($_ =~ /^\s*up\s+route\s+add\s+(-net|-host)\s+(.*)\s+gw\s+(.*)\s+dev\s+$currentInterfaceName\s*/) {
                # look for static routes in the form
                #     up route add <-net|-host> <destination> gw <gateway> dev <interface>
                my $route = Route->new();
                $route->type($1);
                $route->destination($2);
                $route->gateway($3);

                $currentInterface->add_route($route);

            }elsif ($_ =~ /^\s*route\s+(-A)\s+inet6\s+add\s+(.*)\s+gw\s+(.*)\s+dev\s+$currentInterfaceName\s*/) {
                # look for static routes in the form
                #     route -A inet6 add  <destination> gw <gateway> dev eth0
                my $route = Route->new();
                $route->type("");
                $route->destination($2);
                $route->gateway($3);

                $currentInterface->add_route($route);
            }elsif ($_ =~ /^\s*down\s+route.*/) {
                # ignore 'down route' lines (one will always exist per up route)
            }elsif ($_ =~ /^\s*address\s+.+:.+\/.+\s*/) {
                # look for address entry with ':' present and prefixlength appended after '/'
		        my @add = split('/', $_);
			    if ($add[0] =~ /^\s*(\S+)\s+(.*)\s*/) {
			        $currentInterface->set_option($1, $2);	
			    }
			    $add[1]=~ s/\s+//;
			    $currentInterface->set_option("prefixlength", $add[1]);
            } elsif ($_ =~ /^\s*(\S+)\s+(.*)\s*/) {
                # Look for other various options  (dns-nameservers, dns-domain, dns-search, etc)
                $currentInterface->set_option($1, $2);
            }
        }

    }

    close(FH);
    return $interfaces;
}

=item B<write_interfaces_file>

    Function   : Writes the this Interfaces object out to the given interfaces file
    Parameters : path to interfaces file to write

An example interface file for IPv4 static configuration  written by this subroutine might look like this:

    auto lo eth0

    iface eth0 inet static
            address 10.225.150.97
            broadcast 10.225.150.255
            dns-domain gdl.englab.netapp.com
            dns-nameservers 10.60.132.40
            dns-search gdl.englab.netapp.com hq.netapp.com
            gateway 10.225.150.1
            netmask 255.255.255.0
            network 10.225.150.0
            up route add -net 10.0.0.0/16 gw 10.0.0.1 dev eth0
            down route del -net 10.0.0.0/16 gw 10.0.0.1 dev eth0

    iface eth1 inet dhcp

    iface lo inet loopback

An example interface file for IPv6 static configuration  written by this subroutine might look like this:

    auto lo eth0

    iface eth0 inet6 static
            address fd20:8b1e:b255:8567:250:56ff:fe8f:b7f0/64
            gateway fe80::5:73ff:fea0:116

    iface eth1 inet dhcp

    iface lo inet6 loopback

=cut
sub write_interfaces_file {
    my $self = shift;
    my ($interfacesFile) = @_;

    # See comment in parse_interfaces_file for the file format we're writing.

    local *FH;
    open(FH, ">$interfacesFile") or die "Could not open file $interfacesFile for writing: $!\n";

    print FH "auto " . join(" ", @{$self->{AUTO}}) . "\n";
    print FH "\n";

    foreach my $interfaceName (sort keys %{$self->{INTERFACES}}) {
    
        my $interfaceModelArray = $self->{INTERFACES}{$interfaceName};

        foreach my $interfaceModel(@$interfaceModelArray) {

        print FH "iface " . $interfaceModel->name() . " " . $interfaceModel->address_family() . " " . $interfaceModel->config_method() . "\n";

        foreach my $optionName (sort keys %{$interfaceModel->options()}) {
            my $optionValue = $interfaceModel->get_option($optionName);
            if (length($optionValue) != 0) {
                if (($optionName eq "address") && ($interfaceModel->address_family() eq "inet6") && (defined $interfaceModel->get_option("prefixlength"))) {
                    my $preLength = ($interfaceModel->get_option("prefixlength"));
                    print FH "        $optionName $optionValue\/$preLength\n";
                } elsif ($optionName !~ "prefixlen") {
                    print FH "        $optionName $optionValue\n";
                }
            }
            
        }

            foreach my $route (@{$interfaceModel->routes()}) {
                if ($interfaceModel->address_family() eq "inet6") {
                    print FH "        route -A inet6 add " . " " . $route->destination() . " gw " . $route->gateway() . " dev " . $interfaceName . "\n";
                    print FH "        route -A inet6 del " . " " . $route->destination() . " gw " . $route->gateway() . " dev " . $interfaceName . "\n";
                }else {
                    print FH "        up route add " . $route->type() . " " . $route->destination() . " gw " . $route->gateway() . " dev " . $interfaceName . "\n";
                    print FH "        down route del " . $route->type() . " " . $route->destination() . " gw " . $route->gateway() . " dev " . $interfaceName . "\n";
                }
            }

        print FH "\n";
    }

    }
    close(FH);
}

=back

=cut

1; # so the require or use succeeds
