#!/bin/bash

# This file contains common functions for all scripts.
#
# Initialize common data to all scripts, create default
# directories and links if they don't exist.

###############################################################################
#
# Returns 0 if the shell is on the vm console
# Returns 1 otherwise (e.g. if the shell is via ssh, or non-interactively with redirected stdin)
#
###############################################################################

. /lib/lsb/init-functions

servicestop=0
servicestart=0

MAINT_HOME="/home/maint"

function onVmConsole()
{
    # a tty of /dev/tty indicates we're running interactively on the vm console
    tty | grep "/dev/tty" > /dev/null 2>&1
}

#
# Press ENTER
function anykey() {
    read -s -r -n1 -p " Press ENTER to continue." input
    echo
}

#
# Press ENTER to exit
function anykeyExit() {
    read -s -r -n1 -p " Press ENTER to exit." input
    echo
}

# Checks available disk space(for second disk), prompts a user if there is not enough
# to proceed and terminates after a keypress.
function checkAvailableSpace() {
    echo
    echo "Checking available disk space."
    echo
    declare -i maxAllowedUsedDiskSpacePercentage=$1
    spaceUsed=`df -h | grep "^/dev/mapper" | awk '{ print $5 }'`
    declare -i spacePercentage=${spaceUsed%%\%}

    if [[ $spacePercentage -gt $maxAllowedUsedDiskSpacePercentage ]]; then
        echo "Unable to perform operation due to available disk space limitations."
        echo "Please add additional storage to this Virtual Machine or contact customer support for assistance."
        anykey
        logOut
    fi
}

# Method to calculate used space. Rreturns the used disk space
function calculateUsedSpace() {
    echo
    echo "Calculating used disk space"
    echo
    spaceUsed=`df -h | grep "^/dev/mapper" | awk '{ print $5 }'`
    declare -i spacePercentage=${spaceUsed%%\%}

    return $spacePercentage
}

#mounts the cdrom
function mountCdrom()
{
    mount /cdrom &> /dev/null
}


#Unmounts the cdrom if possible
function umountCdrom()
{
    umount /cdrom &> /dev/null
}

#Determines if the cdrom is currently mounted
# return 0 if mounted
# return 1 if not mounted
function isCdromMounted()
{
    mount | grep cdrom &> /dev/null
}

#Determines if it is possible to mount the cdrom by checking if a "cd is in the drive".
# return 0 if can mount
# return 1 if cannot mount
function canMountCdrom() {
    # Attempt to read the first byte of the cdrom device
    # This will fail with an exit code of 1 if the cd is not in the drive
    head --bytes=1 /dev/cdrom &> /dev/null
}

#Calls clean up methods and exits the script
function logOut()
{
    umountCdrom

    echo
    exit
}

#
#Method for displaying an editable field
#
function displayEditField()
{
    # Input parameters:
    #  $1 = prompt
    #  $2 = current setting
    #  $3 = hidden or not
    prompt=$1
    current=$2
    hidden=$3
    new=$current

    # Place the current position at the end of the current
    currpos=${#current}

    # Now lets let the user input what they want
    echo -n "$prompt $current"
    while [[ 1 ]]
    do
        # Read one character at a time to look throught the data
        read -sn 1 char

        # If the character is the escape, then read 5 more.  We are
        # trying to remove the ability to use the arrow keys
        key=$( echo $char | od -t x1 | awk '{print $2}' )
        if [[ $key == "1b" ]]
        then
            loop=5
            while [[ $loop != 0 ]]
            do
                read -sn 1 char
                #echo $char
                loop=$((loop-1))
            done
            continue
        fi

        # This is the backspace character
        if [[ $char == "" ]]
        then
            # Need to make sure we do not allow the user to go beyond the
            # current.
            if [[ $currpos -ne 0 ]]
            then
                echo -e -n "\b \b"
                # Need to subtract from the current pos and as well,
                # need to subtract from new
                currpos=$((currpos-1))
                new=${new:0:$currpos}
            fi
        elif [[ $char == "" ]] || [[ $char == "\n" ]] || [[ $char == "" ]]
        then
            g_output=$new
            return 0
        elif [[ $char == "[1;5A" ]] || [[ $char == "[1;5C" ]]
        then
            continue
		elif [[ $char == "*" ]]
		then
			continue
        else
            # We need to add a new character to the new and update our
            # current position
            if [[ ! "$hidden" ]]
            then
                echo -n $char
            fi
            new=$new$char
            currpos=$((currpos+1))
        fi
    done
}

# Takes 1 parameter, the username to check
# Returns 0 if the user is locked, or 1 if it is not locked.
function userLocked()
{
    # If the account is locked, the output of "passwd -S diag"
    # will be similar to "diag L 08/23/2012 0 99999 7 -1".
    # A value of L in the second field indicates a locked password.

    passwd -S $1 | cut -d ' ' -f 2 | grep "L" >/dev/null 2>&1
}

# Takes 1 parameter, the username to check
# Returns 0 if the user is expired, or 1 if it is not expired.
function userExpired()
{
    # These day values are just numbers, the number of days since the epoch.
    local expireDay=`grep $1 /etc/shadow | cut -d: -f8`

    # If diag has no expiration date, it will be empty.
    if [[ "$expireDay" -eq "" ]] ; then
        return 1
    fi

    local today=`perl -e 'print int(time/(60*60*24))'`
    local daysUntilExpire=`echo $expireDay - $today | bc`

    [[ $daysUntilExpire -lt 0 ]]
}

###############################################################################
#
# Queries the server status.
# Returns the exit code of '$1 status'
# Sets the serverStatus variable to a text description of the status.
#
# @param service
# @returns status return code (0 = running)
###############################################################################
function determineServerStatus() {
    local app=$1
    sudo /usr/sbin/service $app status > /dev/null 2>&1
    local serverStatusReturnCode="$?"
    case $serverStatusReturnCode in
        0)
            echo "Running"
            ;;
        *)
            echo "Not running"
            ;;
    esac
    
    return $serverStatusReturnCode
}

###############################################################################
#
# Start or stop the service and dependencies
#
# @param service (or 'all' for all app.d services)
# @param action (start or stop)
# @returns action result (0 for success)
#
###############################################################################
function serverStartStop() {
    local app=$1
    local action=$2
    
    if [ $app == "all" ]; then
        local apps=$(cd ${MAINT_HOME}/app.d; ls)
        local status=""
        for app in $apps ; do
            sudo /usr/sbin/service $app $action
        done
    else
        sudo /usr/sbin/service $app $action
        local serverStatusReturnCode="$?"
        case $serverStatusReturnCode in
            0)
                echo "Command succeeded." 
                ;;
            *)
                echo "Command failed."
                ;;
        esac
	echo
    fi
    
    return $serverStatusReturnCode
}
