#!/usr/bin/perl

# For OVF parsing
use XML::Parser;

# For option parsing
use Getopt::Long;


###############################################################################
# Find the network ethernet interface
###############################################################################
sub getEthernetInterface() {
     # Find the first ethernet interface
     my $interfaceName = "eth0";
     open(FH, "/bin/ls -1 /sys/class/net |") or die "Could not run /bin/ls: $?\n";
     while(<FH>) {
          chomp;
          if ($_ =~ /^(e.*)/) {
              $interfaceName = $1;
              last;
          }
     }
     close(FH);
     return $interfaceName;
}

$ETHERNET_INTERFACE = getEthernetInterface();

# This script is used to verify that networking is configured properly. It verifies the network configuration by the following:
#
# 0) Discover the network configuration from OVF environment
# 1) Discover the IP address of eth0 from ifconfig
# 2) Discover the gateway address from route
# 3) Ping the gateway
#
# If any of these steps fail, the network is considered not working and the user is prompted to configure static networking.
# After the prompted values are confirmed, the configuration is applied and the network configuration is tested again.

# Pattern with capturing group for matching a valid IP address
$IP_PATTERN = "((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))";


$IP6_PATTERN = qr/^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$/;

# Pattern with capturing group for matching a valid fully qualified domain name hostname
$FQDN_PATTERN = "((?:(?:[a-zA-Z0-9](?:[a-zA-Z0-9\\-]*[a-zA-Z0-9])?)\\.)+(?:[A-Za-z](?:[a-zA-Z0-9\\-]*[a-zA-Z0-9])?))";

# Pattern with IP's capturing group for finding the gateway address from the route command output
$ROUTE_GATEWAY_PATTERN = "0\.0\.0\.0\\s*$IP_PATTERN\\s*.*UG.*$ETHERNET_INTERFACE";

# Pattern for determining whether the ping was successful from the ping command output
$PING_PATTERN = "1 received";

$NETWORK_CONFIG_CHANGED=0;
$UPDATE_RC_CMD="/usr/sbin/update-rc.d";
$SERVICE_CMD="/usr/sbin/service";
$UPGRADEMODE="off";
#
# Asks the user a yes/no question
#
# @param Question for user
# @param Default value if ENTER is pressed with no value
#
# @return 1 for yes, 0 for no
sub user_confirm {
    my $question = shift @_;
    my $default = shift @_;

    while(1) {
        print "$question [$default]: ";
	select()->flush();
        my $answer = <STDIN>;
        chomp $answer;

        #if they didnt enter anything, use the default
        if($answer eq "") {
            $answer = $default;
        }

        # figure out if they entered yes or no
        if ($answer =~ /^(y|yes)$/i) {
            return 1;
        } elsif ($answer =~ /^(n|no)$/i) {
            return 0;
        } else {
            print "Please answer yes or no.\n";
        }
    }
}

#
# Print usage and exit
sub doUsage() {
    print "\nusage: firstboot-network-config.pl [help|init|check|config]\n";
    print "Check and configure network.  No parameters for network check.\n";
    print "\t-help - Print this output and exit.\n";
    print "\t-init - Start network configuration from OVF data import.\n";
    print "\t-config - Start network configuration from user input.\n";
    print "\t-check - Check and print current network configuration.\n";
    print "\t         (Configure if errors detected.)\n";
    exit(99);
}

#
# Parse arguments and return operation mode or fail.
# check - Perform check of setup and prompt user for action if an error is detected.
# config - Prompt user for configuration.
# init - Attempt to read initial config from OVF and prompt user if problem is detected.
#
# @param - none
# @return mode string: 'init', 'check' or 'config'
sub processArguments {
    my $result = "check"; # Default mode 

    # Local variables to hold the arg values
    my ($help, $init, $check, $config, $upgrade);

    # Parse command line and get values
    GetOptions('help'       => \$help,
               'init'       => \$init,
               'check'      => \$check,
               'config'     => \$config,
               'upgrade'    => \$upgrade
    );
    
    # Check in order to give more involved arguments precidence.
    my $num_args = $#ARGV + 1;
    if ($num_args > 0 || $help) {
        doUsage;
    }
    elsif ($config) {
        $result = "config";
    }
    elsif ($init) {
        $result = "init";
    }
    elsif ($upgrade) {
        $result = "upgrade";
    } 
    else {
        return $result;
    }

    return $result;
}

#
# This method will check the link status of eth0.
#
#     "Link detected: no"
#     "Link detected: yes"
#
# @return 1 means link detected 0 means no link detected
#
sub checkLinkStatus {
   my $result = 0; # Default to failed
   print "\nChecking network link status. Please wait...\n";

   # Execute the ethtool command to search for the link status
   open(ETHTOOL, "/sbin/ethtool $ETHERNET_INTERFACE|") or die "Failed to execute ethtool";
   while (<ETHTOOL>) {
      # Look for the link status
      if ($_ =~ m/Link detected: yes/) {
          $result = 1;
          break;
      }
   }
   close(ETHTOOL);

    if ( $result ) {
        print "Link has been detected\n";
    } else {
        print "Link was not detected\n";
    }

   return $result;
}

#
# Check that networking is up and running successfully by detecting the system's
# IP address, and pinging the gateway.
#
# @return 0 if the network is not working, 1 if it is working
#
sub checkNetwork {
   my $result = 0; # Default to failed

   print "\nChecking network configuration. Please wait...\n";


   # Execute the ifconfig eth0 command to search for the IP4 address
   open(IFCONFIG, "ifconfig $ETHERNET_INTERFACE|") or die "Failed to execute ifconfig";
   my $ip4Address = "";
   while (<IFCONFIG>) {
      # Look for the IPV4 address
      if ($_ =~ /inet $IP_PATTERN/) {
          $ip4Address = $1;
          break;
      }
      if ($_ =~ /inet addr:$IP_PATTERN/) {
          $ip4Address = $1;
          break;
      }
   }
   close(IFCONFIG);

   # Execute the ifconfig eth0 command to search for the IP6 address
   open(IFCONFIG, "/sbin/ifconfig $ETHERNET_INTERFACE |grep 'inet6'|") or die "Failed to execute ifconfig";
   my $ip6Address = "";
   while (<IFCONFIG>) {
      # Look for the IPv6 address
      if ($_ =~ /inet6/) {
          my @temp = split /\s+/, $_;
          if ($temp[6] =~ m/<global>/) {
              $ip6Address = $temp[2];
              break;       
          }
      }
   }
   close(IFCONFIG);

   # If the address was not found, the network is invalid
   if ($ip4Address eq "" and $ip6Address eq "") {
       print "No valid network address found.\n";
       return $result;
   }


   if($ip4Address ne ""){

    # Get the gateway from the route and then ping it
     open(ROUTE, "/sbin/route -n |") or die "Failed to execute route";
     my $ip4gateway = "";
     while (<ROUTE>) {
         # Look for the Gateway address
         if ($_ =~ /$ROUTE_GATEWAY_PATTERN/) {
       $ip4gateway = $1;
       break;
         }
     }
     close(ROUTE);

     # check if we got a gateway address
     if ($ip4gateway eq "") {
         print "No default route has been configured.\n";
         return $result;
     }

     # Ping the gateway address to see if it responds
     open(PING, "ping -c 1 -w 1 $ip4gateway |") or die "Failed to execute ping";
     my $success = 0;
     while (<PING>) {
         # Look for reponse
         if ($_ =~ /$PING_PATTERN/) {
       $success = 1;
       break;
         }
     }
     close(PING);

     # check if we got a response
     if ($success == 0) {
         print "Unable to reach gateway at '$gateway'.\n";
         return $result;
     } else {
         print "Network gateway reachable for IPv4 address.\n";
     }


   }

   if($ip6Address ne ""){
      open(ROUTE, "/sbin/route -6 |") or die "Failed to execute route";
       my $ip6gateway = "";
       while (<ROUTE>) {
           # Look for the Gateway address
          if ($_=~ /[\[::\]]\/0.*UG*.*$ETHERNET_INTERFACE/) {
            my @temp = split /\s+/, $_;
            $ip6gateway = $temp[1];
            break;
          }
       }
       close(ROUTE);

       # check if we got a gateway address
       if ($ip6gateway eq "") {
           print "No default route has been configured.\n";
           return $result;
       }
       # Ping the gateway address to see if it responds
       open(PING, "ping6 -I $ETHERNET_INTERFACE -c 1 $ip6gateway |") or die "Failed to execute ping";
       my $success = 0;
       while (<PING>) {
           # Look for reponse
           if ($_ =~ /$PING_PATTERN/) {
         $success = 1;
         break;
           }
       }
       close(PING);

       # check if we got a response
       if ($success == 0) {
           print "Unable to reach gateway at '$gateway'.\n";
           return $result;
       } else {
           print "Network gateway reachable for IPv6 address.\n";
       }


   }
 
   return 1;
}

#
# Check that networking is up and running successfully by detecting the system's
# IP address, and pinging the gateway  for address family.
#
# @return inet6 if the network is available for IPv6, inet if the network is available for IPv4
#
sub checkNetworkAndGetAddressFamily(){
   # Execute the ifconfig eth0 command to search for the IP6 address
   open(IFCONFIG, "/sbin/ifconfig $ETHERNET_INTERFACE |grep 'inet6'|") or die "Failed to execute ifconfig";
   my $ip6Address = "";
   while (<IFCONFIG>) {
      # Look for the IPv6 address
      if ($_ =~ /inet6/) {
          my @temp = split /\s+/, $_;
          if ($temp[6] =~ m/<global>/) {
              $ip6Address = $temp[2];
              break;       
          }
      }
   }
   close(IFCONFIG);
   if($ip6Address ne ""){
      open(ROUTE, "/sbin/route -6 |") or die "Failed to execute route";
       my $ip6gateway = "";
       while (<ROUTE>) {
           # Look for the Gateway address
          if ($_=~ /[\[::\]]\/0.*UG*.*$ETHERNET_INTERFACE/) {
            my @temp = split /\s+/, $_;
            $ip6gateway = $temp[1];
            break;
          }
       }
       close(ROUTE);
       # check if we got a gateway address
       if ($ip6gateway ne "") {
          # Ping the gateway address to see if it responds
           open(PING, "ping6 -I $ETHERNET_INTERFACE -c 1 $ip6gateway |") or die "Failed to execute ping";
           my $success = 0;
           while (<PING>) {
               # Look for reponse
               if ($_ =~ /$PING_PATTERN/) {
                 $success = 1;
                 break;
               }
           }
           close(PING);
           # check if we got a response
           if ($success != 0) {
              return "inet6";
           } 
       }
   }
  open(IFCONFIG, "ifconfig $ETHERNET_INTERFACE|") or die "Failed to execute ifconfig";
   my $ip4Address = "";
   while (<IFCONFIG>) {
      # Look for the IPV4 address
      if ($_ =~ /inet $IP_PATTERN/) {
          $ip4Address = $1;
          break;
      }
      if ($_ =~ /inet addr:$IP_PATTERN/) {
          $ip4Address = $1;
          break;
      }
   }
   close(IFCONFIG);
   if($ip4Address ne ""){
    # Get the gateway from the route and then ping it
     open(ROUTE, "/sbin/route -n |") or die "Failed to execute route";
     my $ip4gateway = "";
     while (<ROUTE>) {
         # Look for the Gateway address
         if ($_ =~ /$ROUTE_GATEWAY_PATTERN/) {
             $ip4gateway = $1;
             break;
         }
     }
     close(ROUTE);
     # check if we got a gateway address
     if ($ip4gateway ne "") {
         # Ping the gateway address to see if it responds
         open(PING, "ping -c 1 -w 1 $ip4gateway |") or die "Failed to execute ping";
         my $success = 0;
         while (<PING>) {
             # Look for reponse
             if ($_ =~ /$PING_PATTERN/) {
                 $success = 1;
                 break;
             }
         }
         close(PING);
         # check if we got a response
         if ($success != 0) {
             return "inet";
         }
      }
   }
   # If the address was not found, the network is invalid
   if ($ip4Address eq "" and $ip6Address eq "") {
       print "No valid network address found.\n";
   }
   # check if we got a response
   if($success4 != 0 && $success6 !=0 ) {
     return "both";
   }elsif($success4 != 0) {
    return "inet";
   }elsif($success6 != 0) {
    return "inet6";
     }
  }

#
# Determine current network configuration (static or DHCP)
#
# @return 1 if the network is static , 0 if dhcp
#
sub isStaticNetworkConfigConfig {
   my $result = 1; # Default to static config

   # Look for the eth0 in the interfaces for and see how it is configured
   open(IFTYPE, "cat /etc/network/interfaces | grep $ETHERNET_INTERFACE | grep iface |") or die "Failed to find network file";
   while (<IFTYPE>) {
      # What type is the interface set for dhcp or static?
      if ($_ =~ m/dhcp/) {
          $result = 0;
      }

      if ($_ =~ m/auto/) {
          $result = 0;
      }
   }
   close(IFTYPE);

#if ( $result == 0 ) {
#       print "IF type is dhcp\n";
#   } else {
#       print "IF type is static\n";
#   }

   return $result;
}

#
# Function to remove whitespaces before and after a string
#
sub trim($) {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

#
# Prompt the user to enter an IPv4 or IPv6 address.
#
# @param promptText - the text to use as the prompt for the IP address
# @param required - to see if we require an entry for this.
# @return the valid IP address
#
sub promptForIP {
   my ($promptText, $isRequired, $ipfamily) = @_;

   my $value = "";
   while (!$value) {
      print "$promptText: ";
      select()->flush();
      my $input = <STDIN>;
      chomp($input);
      $input = trim($input);
      if ( $input eq "" && $isRequired eq "false" ) {
          return $input;
      } else {
          
          if ($ipfamily == 4) {
              my $res = validateIP4($input);
              if($res == 0){

                print "Invalid IP address. Please enter a valid IPv4 address.\n";

             }else {
                $value = $input;
            }
          } elsif ($ipfamily == 6) {
             my $res = validateIP6($input);
              if($res == 0){

                print "Invalid IP address. Please enter a valid IPv6 address.\n";

             }else {
                $input = trimSquareBrackets($input);
                $value = $input;
            }
          } elsif ($ipfamily == 0) {
             my $res = validateIP6($input);
              if($res == 0){

                  my $res = validateIP4($input);
                  if($res == 0){

                    print "Invalid IP address. Please enter a valid IPv6 or IPv4 address.\n";

                 }else {
                    $value = $input;
                } 

             }else {
                $value = $input;
            }
          } 
      }
   }
   return $value;
}


sub promptForPrefixLength {
   my ($promptText, $isRequired) = @_;

   my $value = "";
   while (!$value) {
      print "$promptText: ";
      select()->flush();
      my $input = <STDIN>;
      chomp($input);
      $input = trim($input);
      if ( $input eq "" && $isRequired eq "false" ) {
          return $input;
      } else {
         $value = $input;
      }
   }
   return $value;
}



sub promptForAddressFamily {
  my $addressfamily;
  my $input;
  while (1) {
      print "\nDo you want to configure IPv6 static settings? (y/n):\n";
      select()->flush();
      $input= <STDIN>;
      if($input == 'y'){
        $addressfamily ="IPv6";
        last;
      } elsif($input == 'n'){
        $addressfamily ="IPv4";
        last;
      }
  }
  return $addressfamily;

}

#
# Validate that the IP address matches the expected IPv4 or IPv6 format. 
#
# @param ip - the ip to validate
# @return 1 if the IPv4 is valid, 0 if not
#
sub validateIP4 {
   my ($ip) = @_;

   if ($ip =~ /$IP_PATTERN/) {
      if ($ip eq $1) {
         return 1;
      }
   }
   return 0;
}


#
# Validate that the IP address matches the expected IPv4 or IPv6 format. 
#
# @param ip - the ip to validate
# @return 1 if the IPv6 is valid, 0 if not
#
sub validateIP6 {
   my ($ip) = @_;
   if ($ip =~ /$IP6_PATTERN/) {
      if ($ip eq $1) {
        return 1;
      }
   }
   return 0;
}


#
# Check for ip address family. 
#
# @param ip - the ip to validate
# @return 4 if the IPv4 is valid,6 if the IPv6 is valid, 0 if not
#
sub checkIpFamliy {
   my ($ip) = @_;
   if ($ip =~ /$IP_PATTERN/) {
      if ($ip eq $1) {
         return 4;
      }
   }
   if ($ip =~ /$IP6_PATTERN/) {
      if ($ip eq $1) {
        return 6;
      }
   }
   return 0;
}

#
# Disable or Enable IPv6 stack in UA 
# @param 0 - Enable, 1 - Disable
#

sub enableDisableIPv6 {

    my $optn = shift;

    my $SYSCTLFILE = "/etc/sysctl.conf";
    
    my $settingsAlreadyExists;

    open(FILE,"$SYSCTLFILE");
    $settingsAlreadyExists = (grep{/net.ipv6.conf.all.disable_ipv6/} <FILE>);
    close FILE;

    if($settingsAlreadyExists){
      open(FILE, "<$SYSCTLFILE") || die "File not found";
      my @lines = <FILE>;
      close(FILE);

      my @newlines;
      foreach(@lines) {
          if($optn  == 0){
                $_ =~ s/net.ipv6.conf.all.disable_ipv6 = 1/net.ipv6.conf.all.disable_ipv6 = 0/g;
                $_ =~ s/net.ipv6.conf.default.disable_ipv6 = 1/net.ipv6.conf.default.disable_ipv6 = 0/g;
                $_ =~ s/net.ipv6.conf.lo.disable_ipv6 = 1/net.ipv6.conf.lo.disable_ipv6 = 0/g; 
            } else{

              $_ =~ s/net.ipv6.conf.all.disable_ipv6 = 0/net.ipv6.conf.all.disable_ipv6 = 1/g;
              $_ =~ s/net.ipv6.conf.default.disable_ipv6 = 0/net.ipv6.conf.default.disable_ipv6 = 1/g;
              $_ =~ s/net.ipv6.conf.lo.disable_ipv6 = 0/net.ipv6.conf.lo.disable_ipv6 = 1/g; 
            }
         push(@newlines,$_);
      }

      open(FILE, ">$SYSCTLFILE") || die "File not found";
      print FILE @newlines;
      close(FILE);

    } else{
      open(my $fd, ">>$SYSCTLFILE");
      print $fd "\nnet.ipv6.conf.all.disable_ipv6 = $optn";
      print $fd "\nnet.ipv6.conf.default.disable_ipv6 = $optn";
      print $fd "\nnet.ipv6.conf.lo.disable_ipv6 = $optn";
      close($fd);

    }

    
    open(CMD, "sysctl -p |");
    @cmdlines = <CMD>;
    close(CMD);

}


#
# Prompt the user to enter the static networking configuration
#
# @return %config - map containing the network configuration settings
#
sub promptForNetworkConfig {
   my %config;
   my $ipAddress;
   print "\n";
   print "Static Configuration\n";
   print "--------------------\n";
   $config{ipAllocPolicy} = "STATIC";
   $config{hostName} =       promptForHostName("Host name            ");
   $ipaddress = promptForIP("Host IP address      ", "true", 0);
   $config{ipAddress} =      $ipaddress;
   $isip4or6 = checkIpFamliy($ipaddress);
   if($isip4or6 == 0){
      promptForIP("Host IP address      ", "true", 0);
   }elsif($isip4or6 == 6){
      $config{addressfamily} = "IPv6";
      $config{prefixlength} =        promptForPrefixLength("Prefix length             ", "true");
      $config{gateway} =        promptForIP("Gateway              ", "true", 6);
      $config{primaryDNS} =     promptForIP("Primary DNS address  ", "false", 6);
      $config{secondaryDNS} =   promptForIP("Secondary DNS address", "false", 6);
    } else{
      $config{addressfamily} = "IPv4";
      $config{netMask} =        promptForIP("Net mask            ", "true", 4);
      $config{gateway} =        promptForIP("Gateway              ", "true", 4);
      $config{primaryDNS} =     promptForIP("Primary DNS address  ", "false", 4);
      $config{secondaryDNS} =   promptForIP("Secondary DNS address", "false", 4);
    }
   

   %config = calculateDerivedConfigValues(%config);

   $config{searchDomains} = promptForAdditionalDomains(%config);

   return %config;
}

#
# Prompt the user to enter the host name of the system. 
#
# @param promptText - the text to use as the prompt for the host name
# @return the valid fully qualified host name
#
sub promptForHostName {
   my ($promptText) = @_;

   while (1) {
      print "$promptText: ";
      select()->flush();
      my $input = <STDIN>;
      chomp($input);
      if(validateHostname($input)) {
          return $input;
      } else {
          print "That does not appear to be a valid hostname\n";
      }
   }
}

#returns true if input arg is a valid hostname
sub validateHostname {
    my $hostname = $_[0];

    if ($hostname =~ /[^-A-Za-z0-9]/ || length($hostname) > 63
        || length($hostname) <= 0) {
	return 0;
    }
    return 1;
}

#
# Validate that the FQDN host name matches the expected format. The host name 
# should be in the form of '<hostname>.<domain>' with as many dotted segments 
# in the domain portion as necessary.
#
# @param hostName - the name to validate
# @return 1 if the hostName is valid, 0 if not
#
sub validateFqdnHostName {
   my ($hostName) = @_;
   
   if ($hostName =~ /$FQDN_PATTERN/) {
       if ($hostName eq $1) {
          return 1;
       }
   }
   return 0;
}

#
# Validate if the specified list of search domains are valid
#
# @param domains - space separated list of search domains
# @return 1 = valid, 0 = invalid, @searchDomains.
#
sub validateSearchDomains {
    my ($domainlist) = @_;

    my $valid = 1;
    my $searchDomains = "";

    if ($domainlist ne "") {
	# validate the domains
	@domains = split(/[,\s]+/, $domainlist);
	foreach $domain (@domains) {
	        # Strip off extra whitespace
	        $domain =~ s/^\s+//;
	        $domain =~ s/\s+$//;
            if (!validateFqdnHostName($domain)) {
                print "Invalid search domain: $domain\n";
                $valid = 0;
		last;
            } else {
		# If domain was valid, append it to the search domains value
		$searchDomains .= ' ' . $domain;
            }
	}
    }

    # use substr to strip space at front of string
    return($valid, substr($searchDomains, 1));
}


#
# Prompt the user to optionally specify additional search domains.
#
# @param %config - the configuration values map
# @return the search domain string, separated by spaces between the domains
#
sub promptForAdditionalDomains {
   my (%config) = @_;

   my $searchDomains = "";

   my $valid = 0;

   while (!$valid) {
      print "Please enter in all search domains separated by spaces (can be left blank):\n";
      select()->flush();
      my $input = <STDIN>;
      chomp($input);

      # if additional domains were specified
      ($valid, $searchDomains) = validateSearchDomains($input);
   }

   return $searchDomains;
}

#
# Calculate the derived configuration values from the config values the user entered at the prompts.
# The derived values include the network, broadcast, hostName and domain values in the config map.
#
# @param %config - the configuration values map
# @return %config - the configuration values map with the derived values filled in
#
sub calculateDerivedConfigValues {
   my (%config) = @_;

   # Calculate the network and broadcast values
   
   # Split the IP address and net mask addresses by . into 4 string segments
   my @ipSegments = split /\./, $config{ipAddress};
   my @netMaskSegments = split /\./, $config{netMask};

   my $network;
   my $broadcast;

   if($config{ipAllocPolicy} eq "IPv6"){

    #TODO: calculate network address

    $config{network} = "";

  }else{

      for ($i = 0; $i < 4; $i++) {
      # Add zero to the strings to force conversion to a number
      $ipSegment = $ipSegments[$i] + 0;
      $netMaskSegment = $netMaskSegments[$i] + 0;

      # Bitwise AND the segments together to produce the network segment
      $netSegment = $ipSegment & $netMaskSegment;

      # Bitwise OR the address segment with the inverse of the mask segment to produce the broadcast segment
      $broadcastSegment = $ipSegment | ($netMaskSegment ^ 0xFF);

      if ($i == 0) {
         $network = $netSegment;
         $broadcast = $broadcastSegment;
      } else {
         $network = $network . '.' . $netSegment;
         $broadcast = $broadcast . '.' . $broadcastSegment;
      }
   }

   # Place the calculated values into the config map
   $config{network} = $network;
   $config{broadcast} = $broadcast;

   


  }

   return %config;
}

#
# Display the network configuration and prompt the user to confirm the
# settings are correct.
#
# @param %config - map of the network configuration
#
sub displayConfig {
   my (%config) = @_;

   print "\n\n";
   print "Selected IP policy    : $config{ipAllocPolicy}\n";
   print "Selected host name    : $config{hostName}\n";
   print "Selected IP           : $config{ipAddress}\n";
   if($config{addressfamily} eq "IPv6"){
    print "Selected prefix length     : $config{prefixlength}\n";
   }else{
    print "Selected net mask     : $config{netMask}\n";
   }
   
   print "Selected gateway      : $config{gateway}\n";
   print "Selected primary DNS  : $config{primaryDNS}\n";
   print "Selected secondary DNS: $config{secondaryDNS}\n";
   print "Search domains        : $config{searchDomains}\n";
   if($config{addressfamily} eq "IPv6"){

      print "Calculated network    : $config{network}\n";
     

    }else{
      print "Calculated network    : $config{network}\n";
      print "Calculated broadcast  : $config{broadcast}\n";
    }
   
   
}

#
# Display the network configuration and prompt the user to confirm the
# settings are correct.
#
# @param %config - map of the network configuration
# @return 1 if the user confirmed the configuration, 0 if the user 
#     rejected the configuration
#
sub confirmConfig {
   my (%config) = @_;

   displayConfig(%config);
   return user_confirm("Are these values correct?", 'y');
}

#
# Modify the system to reflect the static network configuration and 
# restart networking using the new configuration.
#
# @param %config - map of the new static network configuration
#
sub applyStaticNetworkConfig {
   my (%config) = @_;

   my $timestamp = time();

   # Update the hostname file
   system("cp", "/etc/hostname", "/etc/hostname.$timestamp.bak");
   open(HOSTNAME, ">/etc/hostname") or die "Failed to open /etc/hostname for writing";
   print HOSTNAME "$config{hostName}\n";
   close(HOSTNAME);

   # Update the hosts file
   open(HOSTS, "/etc/hosts") or die "Failed to read from /etc/hosts";
   @hostLines = <HOSTS>;
   close(HOSTS);

   system("cp", "/etc/hosts", "/etc/hosts.$timestamp.bak");
   open(HOSTS, ">/etc/hosts") or die "Failed to open /etc/hosts for writing";
   # Add localhost info

   if($config{addressfamily} eq "IPv6"){

      enableDisableIPv6(0);

      print HOSTS "::1\tlocalhost\n";

      # Add any lines from the original hosts file that don't refer to localhost (::1) IPv6
     foreach $hostLine (@hostLines) {
        if (index($hostLine, "::1") < 0) {
           print HOSTS $hostLine;
        }
     }
     close(HOSTS);

    } else{

      enableDisableIPv6(1);

      print HOSTS "127.0.0.1\tlocalhost\n";

        # Add any lines from the original hosts file that don't refer to localhost (127.0.0.1) IPv4
       foreach $hostLine (@hostLines) {
          if (index($hostLine, "127.0.0.1") < 0) {
             print HOSTS $hostLine;
          }
       }
       close(HOSTS);

    }

   # Update the host name on the live system
   system("/bin/hostname", $config{hostName});

   # Stop networking (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/networking stop |");
   @cmdlines = <CMD>;
   close(CMD);

   # Read the existing interfaces file
   open(INTERFACES, "/etc/network/interfaces") or die "Failed to read from /etc/network/interfaces";
   @interfacesLines = <INTERFACES>;
   close(INTERFACES);

   system("cp", "/etc/network/interfaces", "/etc/network/interfaces.$timestamp.bak");
   open(INTERFACES, ">/etc/network/interfaces") or die "Failed to open /etc/network/interfaces for writing";
   # Rewrite the interfaces file using the existing lines, substituting the eth0 configuration line(s) with 
   # the new static configuration.
   my $inEth0 = 0;
   foreach $interfacesLine (@interfacesLines) {
      # If this line is the start of the eth0 block
      if ($interfacesLine =~ /^iface $ETHERNET_INTERFACE/) {
         # Toggle the block marker on and skip printing the line
         $inEth0 = 1;
         next; # Continue the foreach loop
      }
      # Look for whitespace at the beginning of the line to indicate whether we're still in the
      # eth0 block
      if ($inEth0 && $interfacesLine !~ /^\s/) {
         # Detected the end of the eth0 block. Insert the new configuration lines, then toggle the block
         # marker off
         my $inetfamily = ($config{addressfamily} eq "IPv6") ? "inet6" : "inet";

         print INTERFACES "iface $ETHERNET_INTERFACE $inetfamily static\n";

         if(($config{addressfamily} eq "IPv6") && ( $config{prefixlength} ne "")) {
             print INTERFACES "\taddress $config{ipAddress}\/$config{prefixlength}\n";
         } else {
             print INTERFACES "\taddress $config{ipAddress}\n";
         }
          
         if ( $config{network} ne "" ) {
             print INTERFACES "\tnetwork $config{network}\n";
         }
         
         if($config{addressfamily} ne "IPv6"){
             if ( $config{netMask} ne "" ) {
                 print INTERFACES "\tnetmask $config{netMask}\n";
             }
             if ( $config{broadcast} ne "" ) {
                 print INTERFACES "\tbroadcast $config{broadcast}\n";
             }
         }
         
         if ( $config{gateway} ne "" ) {
             print INTERFACES "\tgateway $config{gateway}\n";
         }
         if ( $config{domain} ne "" ) {
             print INTERFACES "\tdns-domain $config{domain}\n";
         }
         if ( $config{searchDomains} ne "" ) {
             # This allows compatibility with AMT (this change was easier)
             print INTERFACES "\tdns-search $config{searchDomains}\n";
         }

         my $dns_servers = "";

         if ( $config{primaryDNS} ne  "") {
              $dns_servers .= $config{primaryDNS} . " ";
         }

         if( $config{secondaryDNS} != "" ) {
              $dns_servers .= $config{secondaryDNS} . " ";
         }

         if($dns_servers ne "") {
               print INTERFACES "\tdns-nameservers $dns_servers\n";
	 }

         $inEth0 = 0;
      }
      # If we're not in the eth0 block, print the line from the original interfaces file
      if (! $inEth0) {
         print INTERFACES $interfacesLine;
      }
   }
   close(INTERFACES);

    
  # During upgrade restarting of network service is not required, beacasue changes will persist only after the VM reboot
  if($UPGRADEMODE eq "off"){
	   # Start networking with the new configuration (use open/close rather than system to swallow command output)
	   open(CMD, "/etc/init.d/networking start |");
	   @cmdlines = <CMD>;
	   close(CMD);
	   
	   # During upgrade restarting of VMware tools is not required, beacasue changes will persist only after the VM reboot
	   # Restart VMware Tools (use open/close rather than system to swallow command output)
	   open(CMD, "/etc/init.d/vmware-tools restart |");
	   @cmdlines = <CMD>;
	   close(CMD);
	
	   # During upgrade advertisement is not required, beacasue changes will persist only after the VM reboot
	   # Send a ping to advertise ourselves
	   open(CMD, "/bin/ping -c 1 " . $config{gateway} ." |");
	   @cmdlines = <CMD>;
	   close(CMD);
   }
   $NETWORK_CONFIG_CHANGED=1;
}

#
# Setup static network configuration from user values
#
# @param - none
# @return - none
#
sub setupStaticIp {

    # While the network configuration is invalid
    $forceFail = 0;
    while (!$valid) {
        my %config;
        my $confirmed = 0;

        # Prompt for the network configuration and loop until it's confirmed
        while (!$confirmed) {
            %config = promptForNetworkConfig();
            $confirmed = confirmConfig(%config);
        }

        # Apply the network configuration, and loop back to testing the network configuration
        print "\n\nApplying network configuration. Please wait...\n";
        applyStaticNetworkConfig(%config);

        # Check the network validity with the new configuration
        $valid = checkNetwork();

        # If the configuration is still invalid, prompt the user if they want to force use these settings.
        # This will allow the user to exit the loop if for some reason the settings are correct, but
        # we still can't ping the gateway.
        if (!$valid) {
            if (user_confirm("\nThe network configuration is still invalid.\nAre you sure you want to use these settings?", 'n')) {
                $valid = 1;
                $forceFail = 1;
            }
        }
    }

    if ( $forceFail ) {
        print "\n";
	print "You have chosen to enable appliance with a possible network\n";
	print "configuration error. If you are unable to access the web interface, or it does not\n";
	print "function correctly, please resolve outstanding networking issues and restart this\n";
	print "appliance.\n";
	print "\n";
        exit(1);
    }
}

#
# Apply network config for DHCP
# @param - none
# @return - none
sub applyDhcpConfig {
    print "\nApplying DHCP network configuration. Please wait...\n";

    enableDisableIPv6(0);

    open (IFACE_FILE, "<", "/etc/network/interfaces");

    my $out_file = "";

   # Stop networking (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/networking stop |");
   @cmdlines = <CMD>;
   close(CMD);

    while(my $line = <IFACE_FILE>) {
        if($line =~ /^iface $ETHERNET_INTERFACE/) {
            while($line = <IFACE_FILE>) {
                if(!($line =~ /^\t/)) {
                    last;
                }
            }
            $out_file .= "iface $ETHERNET_INTERFACE inet dhcp\n" . "iface $ETHERNET_INTERFACE inet6 auto\n";
        }
        $out_file .= $line;
    }

    close (IFACE_FILE);
    open (OUTPUT_FILE, ">", "/etc/network/interfaces");
    print OUTPUT_FILE $out_file;
    close (OUTPUT_FILE);
    
    
  # During upgrade restarting of network service is not required, beacasue changes will persist only after the VM reboot
  if($UPGRADEMODE eq "off"){
   # Start networking with the new configuration (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/networking start |");
   @cmdlines = <CMD>;
   close(CMD);
   
   # During upgrade restarting of VMware tools is not required, beacasue changes will persist only after the VM reboot
   # Restart VMware Tools (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/vmware-tools restart |");
   @cmdlines = <CMD>;
   close(CMD);
  }
   $NETWORK_CONFIG_CHANGED=1;
}

my %netconfig;

#
# OVF environment XML processing handlers
#
sub OVF_start_handler {
    my $p  = shift;
    my $el = shift;
    while (my $key = shift) {
	my $val = shift;
	if ($key == "oe.key" && $val =~ /vplatform/) {
	    my $attr = $val;
	    $key = shift;
	    $val = shift;
	    if ($key == "oe.value") {
		$attr =~ s/vplatform\.//;
		$attr =~ s/\.Vendor_Provider_Appliance//;
		$netconfig{$attr} = $val;
	    }
	}
    }
}

#Check other script files where similar function is added as that also needs modification
sub get_vmtoolsd_path() {
	if (-e "/usr/bin/vmtoolsd") {
		return "/usr/bin/vmtoolsd";
	}
	elsif (-e "/usr/sbin/vmtoolsd") {
		return "/usr/sbin/vmtoolsd";
	}
	else {
		return "error";
	}
}

sub OVF_end_handler { }
sub Default_handler { }

#
# Setup network configuration from OVF values
#
# @param - none
# @return - "dhcp" or "static" or "error"
#
sub setupNetworkFromOVF {
    print "\nRetrieving network configuration from VMware Tools. Please wait...\n";
    
    # Read the ovf environment
    my $vmtools = get_vmtoolsd_path();
    my $ovfConfigCmd = $vmtools . " --cmd 'info-get guestinfo.ovfenv' |";
    my $rc = 0;
    if (-r $vmtools && -x $vmtools) {
        open(OVF, $ovfConfigCmd) or die "Failed to execute vmtools command";
        @ovfenv = <OVF>;
        close(OVF);
        $rc = $?;
        if ($rc ne 0) {
        print "VMware Tools OVF network configuration not found.\n";
        return "error";
        }
    } else {
        print "VMware Tools OVF network configuration not found.\n";
        return "error";
    }

    # parse the OVF xml content
    my $parser = new XML::Parser();
    $parser->setHandlers(Start => \&OVF_start_handler,
			  End => \&OVF_end_handler,
			  Default => \&Default_handler);
    $parser->parse(join("\n", @ovfenv));
    
    # By default SSH will be disabled and user can enable/disable SSH using maintenance console.
    `sudo $UPDATE_RC_CMD ssh disable`;
    `sudo $SERVICE_CMD ssh stop`;
    
    # if DHCP was requested reset config and return
    #if ($netconfig{ipAllocPolicy} eq "DHCP") {
    if ($netconfig{ipAddress} eq "") {
        print "DHCP configuration selected.\n";
        applyDhcpConfig();
	    return "dhcp";
    } else {
        print "Static configuration selected.\n";
    }

    # Check if the received values are ok
    if (!validateHostname($netconfig{hostName})) {
        print "\nHost name '" . $netconfig{hostName} . "' is invalid.\n";
        return "error";
    }

    my $isip4or6 = checkIpFamliy($netconfig{ipAddress});

      if ($isip4or6 == 0) {
            print "IP address '" . $netconfig{ipAddress} . "' is invalid.\n";
            return "error";
      } elsif ($isip4or6 == 6) {

        $netconfig{addressfamily} = "IPv6";

        # Check all the IP values passed in
        if (validateIP6($netconfig{ipAddress}) == 0) {
            print "IP address '" . $netconfig{ipAddress} . "' is invalid.\n";
            return "error";
        }
        
        if ($netconfig{prefixlength} eq "") {
            print "Prefix length '" . $netconfig{prefixlength} . "' is invalid.\n";
            return "error";
        }

        if (validateIP6($netconfig{gateway}) == 0) {
            print "Gateway address '" . $netconfig{gateway} . "' is invalid.\n";
            return "error";
        }
        if (validateIP6($netconfig{primaryDNS}) == 0) {
            print "Primary DNS address '" . $netconfig{primaryDNS} . "' is invalid.\n";
            return "error";
        }
        if (validateIP6($netconfig{secondaryDNS}) == 0) {
            if ($netconfig{secondaryDNS} ne "") {
                print "Secondary DNS address '" . $netconfig{secondaryDNS} . "' is invalid.\n";
                $netconfig{secondaryDNS} = "";
            }
        }

        $netconfig{ipAddress} = trimSquareBrackets($netconfig{ipAddress});
        $netconfig{gateway} = trimSquareBrackets($netconfig{gateway});
        $netconfig{primaryDNS} = trimSquareBrackets($netconfig{primaryDNS});
        $netconfig{secondaryDNS} = trimSquareBrackets($netconfig{secondaryDNS});

    }else{

      $netconfig{addressfamily} = "IPv4";
      if (validateIP4($netconfig{ipAddress}) == 0) {
          print "IP address '" . $netconfig{ipAddress} . "' is invalid.\n";
          return "error";
      }
       if (validateIP4($netconfig{netMask}) == 0) {
            print "Netmask address '" . $netconfig{netMask} . "' is invalid.\n";
            return "error";
      }

      if (validateIP4($netconfig{gateway}) == 0) {
          print "Gateway address '" . $netconfig{gateway} . "' is invalid.\n";
          return "error";
      }
      if (validateIP4($netconfig{primaryDNS}) == 0) {
          print "Primary DNS address '" . $netconfig{primaryDNS} . "' is invalid.\n";
          return "error";
      }
      if (validateIP4($netconfig{secondaryDNS}) == 0) {
          if ($netconfig{secondaryDNS} ne "") {
              print "Secondary DNS address '" . $netconfig{secondaryDNS} . "' is invalid.\n";
              $netconfig{secondaryDNS} = "";
          }
      }
      

    }

    # Check the search domain list
    my ($dom_valid, $searchDomains) = validateSearchDomains($netconfig{searchDomains});
    if (!dom_valid) {
    	return "error";
    } else {
        $netconfig{searchDomains} = $searchDomains;
    }

    # Compute the derived values
    %netconfig = calculateDerivedConfigValues(%netconfig);

    # Apply the network configuration, and loop back to testing the network configuration
    print "\n\nApplying network configuration. Please wait...\n";
    applyStaticNetworkConfig(%netconfig);

    # Check the network validity with the new configuration
    $valid = checkNetwork();

    # If the configuration is invalid, report it to the user and force them to re-enter the values 
    # on the console
    if (!$valid) {
        if (user_confirm("The network configuration has errors. Specified attributes may be invalid.\nAre you sure you want to use these settings?", 'n')) {
            $valid = 1;
            $forceFail = 1;
        } else {
            return "error";
        }
    }

    if ( $forceFail ) {
        print "\n";
        print "You have chosen to enable this appliance with a possible network\n";
        print "configuration error. If you are unable to access the web interface, or it does not\n";
        print "function correctly, please resolve outstanding networking issues and restart this\n";
        print "appliance.\n";
        print "\n";
        exit(1);
    }

    # Display the values that were passed in
    print "Successfully configured the manangement interface using the following attributes. \n";
    displayConfig(%netconfig);

    return "static";
}

#
# Initiate manual config
# @param - none
# @return - none
sub doConfig() {
    $isStaticConfig = isStaticNetworkConfigConfig();
    my $result = "error";
    
    print "\nCurrent network configuration is: ";
    if ($isStaticConfig) {
        print "STATIC\n";
        # We are in static mode, notify the user and then ask if they want to set to DHCP mode.
        if (user_confirm("\nWould you like to change to a DHCP network configuration?", 'y')) {
            applyDhcpConfig();
            $result = "dhcp";
        }
        else {
            print "\nConfiguration unchanged.\n";
            $result = "static";
        }
    }
    else {
        print "DHCP\n";
        # We are in DHCP mode, notify the user and then ask if they want to set to static mode.
        if (user_confirm("\nWould you like to change to a static network configuration?", 'y')) {
            setupStaticIp();
            $result = "static";
        }
        else {
            print "\nConfiguration unchanged.\n";
            $result = "dhcp";
        }
    }
    
    return $result;
}

#
# Replace eth0 by actual ethernet interface in /etc/network/interface
#
sub replaceeth0() {
	open(FILE, "</etc/network/interfaces") || die "File not found";
	my @lines = <FILE>;
	close(FILE);

	my @newlines;
	foreach(@lines) {
   		$_ =~ s/eth0/$ETHERNET_INTERFACE/g;
   		push(@newlines,$_);
	}

	open(FILE, ">/etc/network/interfaces") || die "File not found";
	print FILE @newlines;
	close(FILE);


        open(FILE, "</etc/iptables/rules.v6") || die "File not found";
        my @lines1 = <FILE>;
        close(FILE);

        my @newlines1;
        foreach(@lines1) {
                $_ =~ s/eth0/$ETHERNET_INTERFACE/g;
                push(@newlines1,$_);
        }

        open(FILE, ">/etc/iptables/rules.v6") || die "File not found";
        print FILE @newlines1;
        close(FILE);

        open(FILE, "</etc/iptables/rules.v4") || die "File not found";
        my @lines2 = <FILE>;
        close(FILE);

        my @newlines2;
        foreach(@lines2) {
                $_ =~ s/eth0/$ETHERNET_INTERFACE/g;
                push(@newlines2,$_);
        }

        open(FILE, ">/etc/iptables/rules.v4") || die "File not found";
        print FILE @newlines2;
        close(FILE);
}

##################################################################
# Function to remove [] sqaure brackets before and after a string
##################################################################
sub trimSquareBrackets($) {
  my $ipstring = shift;
  $ipstring = $ipstring =~ tr/[]//dr;
  return $ipstring;
}

##################################################################################################################
# Main execution
##################################################################################################################
# At boot, the appliance's startup messages are supressed by redirecting them to an alternate console /dev/ttyS0
# For the interactive portions of this program, we'll need to explicitly switch STDIN and STDOUT to use the real console at /dev/tty1
sub main() {
    open(STDIN, "</dev/tty1");
    open(STDOUT, ">/dev/tty1");
    
    my $mode = processArguments;

    #Replace eth0
    replaceeth0();
    
    # This cleanup must run outside the build process or bad things will happen to good
    # directories. :-)
    # This network config should occur once under normal circumstances so we also
    # nuke any undesirable build-related directories laying around too.
    system("rm -rf /u /x /usr/software");

    # Clear the screen
    #print (((" " x 80) . "\n") x 25);
    system("clear");
    system("/home/maint/scripts/disableTLSProtocal.pl"); 
	require "/home/maint/scripts/disableTLSProtocal.pl";
    print "\nNETWORK CONFIGURATION\n";

    # Check to see if the interface actually has link detected.  If not then the user must not have
    # setup their network card correctly on the virtual network adapter.
    $hasLink = checkLinkStatus();
    if ($hasLink == 0) {
        print "\n";
        print "The virtual adapter is reporting that it cannot detect a link. This is either \n";
        print "due to a virtual machine configuration issue or network issue.  Please check\n";
        print "the network adapter settings to make sure that the \"Device Status\" is set to:\n";
        print "          o - \"Connected\"\n";
        print "          o - \"Connect at power on\"\n";
        print "\n";
        print "This appliance will not function until this is addressed.\n";
        print "Once resolved, please restart this appliance.\n";
        print "\n";
        print "                    <Please press enter to continue>";
	select()->flush();
        $response = <STDIN>;
        print "\n";
        exit(1);
    }

    # This handles initial configuration (aka. -init)
    # Ask them which method they would like to use
    if ($mode eq "init") {
        # Read the OVF environment
        $config = setupNetworkFromOVF();

        if ($config eq "static") {
            exit(0);
        } elsif ($config eq "dhcp") {
            # Get ready for DHCP configuration
            eval {
                local $SIG{ALRM} = sub { die "timeout\n" };
                print "\nThe virtual machine will attempt to be configured automatically using DHCP.\n";
                print "If you want to manually configure the network interface with a static IP,\n";
                print "press <ENTER> now.";
		select()->flush();
                alarm 10;
                my $test = <STDIN>;
                alarm 0;
            };
            if ($@) {
                print "\nUsing DHCP for network interface configuration\n";
            } else {
                print "\nSetting up network for static IP configuration\n";
                setupStaticIp();
                exit(0);
            }
        } elsif ($config eq "error") {
            print "\n";
            print "No network configuration selection was retrieved during OVF prompts.\n";
            print "This could be due to either an issue with VMware Tools or\n";
            print "network configuration issue.\n";
            print "\n";
            if (!user_confirm("Would you like to correct the configuration?", 'y')) {
                print "\n";
                print "You have chosen to enable the virtual appliance with a possible network\n";
                print "configuration error. If you are unable to access the web interface, or it does not\n";
                print "function correctly, please resolve outstanding networking issues and restart this\n";
                print "appliance.\n";
                print "\n";
                print "Press ENTER to continue.";
		select()->flush();
                $continue = <STDIN>;
                exit(1);
            }
            $config = doConfig();
            if ($config eq "static") {
                setupStaticIp();
                exit(0);
            }
        }
    } # End init
    elsif ($mode eq "config") {
        doConfig();
    } # End config
    elsif ($mode eq "upgrade") {
       	
        $UPGRADEMODE = "on";
       
		my @applianceInetArray;
		if(-e "/var/lib/vplatform/upgrade-network-CONFIG") {
		open(my $fh, "<", "/var/lib/vplatform/upgrade-network-CONFIG");
		while(<$fh>) { 
			chomp;
			push @applianceInetArray, $_;
		} 
		close $fh;
		}
		if ( grep( /^inet$/, @applianceInetArray ) ) {
		enableDisableIPv6(1);
		} elsif(grep( /^inet6$/, @applianceInetArray )){
		enableDisableIPv6(0);
		}
		else{
		applyDhcpConfig();
		}
		exit(0);
    } # End upgrade
   
   if($UPGRADEMODE eq "off"){
    # Start check
    # Getting here means that connection is ok. Now do the initial check of the network validity and exit immediately 
    # if the the network is valid.
    $valid = checkNetwork();
    if ($valid) {
        my @commands = ( "/etc/network/if-up.d/loginmsg",
                         "/etc/network/if-up.d/hosts-file",
                         "sleep 10" );
        my @pids;
        foreach my $cmd( @commands ) {
            my $pid = fork;
            if ( $pid ) {
                # parent process
                push @pids, $pid;
                next;
            }
            # now we're in the child
            system( $cmd );
            exit;            # terminate the child
        }
        wait for @pids;   # wait for each child to terminate
        print "Network configuration appears OK.\n";
        my $rc=0;
        if ($NETWORK_CONFIG_CHANGED) {
            $rc=55;
        }
        exit($rc);
    }
    }else{
        print "Network configuration appears OK.\n";
        my $rc=0;
        if ($NETWORK_CONFIG_CHANGED) {
            $rc=55;
        }
        exit($rc);
    }


    # The only way to truly get here is because we are in DHCP mode and we did not get back
    # a response from the server.  So let the user know the situation and then ask if they 
    # want to set to static mode.  Verify first
    #
    $isStaticConfig = isStaticNetworkConfigConfig();
    if ($isStaticConfig) {
        exit(0);
    }

    # We are in DHCP mode, notify the user and then ask if they want to set to static mode.
    print "\n";
    print "The manangement interface is currently configured for DHCP but the network\n";
    print "could not be configured properly.  This could be due to either a DHCP \n";
    print "configuration issue or virtual machine network adapter configuration.\n";
    print "\n";
    if (!user_confirm("Would you like to setup a static configuration?", 'y')) {
        print "\n";
        print "You have chosen to enable the virtual appliance with a possible network\n";
        print "configuration error. If you are unable to access the web interface, or it does not\n";
        print "function correctly, please resolve outstanding networking issues and restart this\n";
        print "appliance.\n";
        print "\n";
        print "Press ENTER to continue.";
	select()->flush();
        $continue = <STDIN>;

        exit(1);
    }

    setupStaticIp();
    exit(0);
}

main;
