#!/usr/bin/perl

# For OVF parsing
use XML::Parser;

# For option parsing
use Getopt::Long;

# add the current dirname to the lib path so we can find our other modules
use File::Basename;
use lib dirname(__FILE__);
use Common::Text;
use Common::Password;

$MAINT_PASSWORD_KEY = "maint.passwd";

#
# Print usage and exit
sub doUsage() {
    print "\nusage: firstboot-set-maintenance-password.pl [help|init|check|config]\n";
    print "Set MAINTENANCE user password.\n";
    print "\t-help    Print this output and exit.\n";
    print "\t-init    Change maintenance password from OVF data import.\n";
    print "\t-config  Change maintenance password from User input.\n";
    print "\t-check   Nothing.\n";
    exit(99);
}

#
# Parse arguments and return operation mode or fail.
# check - Perform check of setup and prompt user for action if an error is detected.
# config - Prompt user for configuration.
# init - Attempt to read initial config from OVF and prompt user if problem is detected.
#
# @param - none
# @return mode string: 'init', 'check' or 'config'
sub processArguments {
    my $result = "check"; # Default mode

    # Local variables to hold the arg values
    my ($help, $init, $check, $config);

    # Parse command line and get values
    GetOptions(
        'help'   => \$help,
        'init'   => \$init,
        'check'  => \$check,
        'config' => \$config
    );

    # Check in order to give more involved arguments precidence.
    my $num_args = $#ARGV + 1;
    if ($num_args > 0 || $help) {
        doUsage;
    }
    elsif ($config) {
        $result = "config";
    }
    elsif ($init) {
        $result = "init";
    } else {
        return $result;
    }

    return $result;
}

my $maintpwdconfig;

#
# OVF environment XML processing handlers
#
sub OVF_start_handler {
    my $p  = shift;
    my $el = shift;
    while ( my $key = shift ) {
        my $val = shift;
        if ( $key == "oe.key" && $val =~ /vplatform/ ) {
            my $attr = $val;
            $key = shift;
            $val = shift;
            if ( $key == "oe.value" ) {
                $attr =~ s/vplatform\.//;
                $attr =~ s/\.Vendor_Provider_Appliance//;
                $maintpwdconfig{$attr} = $val;
            }
        }
    }
}

#Check other script files where similar function is added as that also needs modification
sub get_vmtoolsd_path() {
	if (-e "/usr/bin/vmtoolsd") {
		return "/usr/bin/vmtoolsd";
	}
	elsif (-e "/usr/sbin/vmtoolsd") {
		return "/usr/sbin/vmtoolsd";
	}
	else {
		return "error";
	}
}

sub OVF_end_handler { }
sub Default_handler { }

sub readMaintPasswordFromOVF {
    print "\nRetrieving maintenance user password from VMware Tools. Please wait...\n";

    # Read the ovf environment
    my $vmtools      = get_vmtoolsd_path();
    my $ovfConfigCmd = $vmtools . " --cmd 'info-get guestinfo.ovfenv' |";
    my $rc           = 0;
    if ( -r $vmtools && -x $vmtools ) {
        open( OVF, $ovfConfigCmd ) or die "Failed to execute vmtools command";
        @ovfenv = <OVF>;
        close(OVF);
        $rc = $?;
        if ( $rc ne 0 ) {
            print "VMware Tools OVF network configuration not found.\n";
            return "error";
        }
    }
    else {
        print "VMware Tools OVF network configuration not found.\n";
        return "error";
    }

    # parse the OVF xml content
    my $parser = new XML::Parser();
    $parser->setHandlers(
        Start   => \&OVF_start_handler,
        End     => \&OVF_end_handler,
        Default => \&Default_handler
    );
    $parser->parse( join( "\n", @ovfenv ) );

    if ( $maintpwdconfig{$MAINT_PASSWORD_KEY} eq "" ) {
        print "No Maint password was given during deployment.\n";
        return "empty";
    }
	
	if ( !validateText($maintpwdconfig{$MAINT_PASSWORD_KEY})) {
	    print "Please use a longer password for maint user.\n";
        return "error";
	}

    return $maintpwdconfig{$MAINT_PASSWORD_KEY};
}

#
# Set maintenance password
# @returns 0 for OK or non-zero for problem
sub setMaintPasswordFromOVF {
    my $message    = readMaintPasswordFromOVF();
    my $retVal         = 0;    
    
    if ($message eq "empty") {
        print "Unable to read maint password from OVF.\n";
        $retVal = 1;
    }
    else {
        print "Error obtaining information from OVF.\n";
        $retVal = 2;
    }
    
    return $retVal;
}

sub doConfig {
    my %config = @_;

    my $password   = "";
    my $maint_user = "";
    my $rc         = 0;

    if ( keys(%config) <= 0 ) {
        %config = promptForMaintPassword();
    }

    $rc = setMaintPassword(%config);

    return $rc;
}

sub promptForMaintPassword {
    my %config;

    $config{$MAINT_PASSWORD_KEY} = $MAINT_PASSWORD;
    
    return %config;
}

sub promptForPassword {
   my ($promptText) = @_;

   while (1) {
      print "$promptText: ";
      my $input = Password::askPassword();
      chomp($input);
      if(validateText($input)) {
          return $input;
      } else {
          print "That does not appear to be a valid password.\n";
      }
   }
}

sub setMaintPassword {
    my %config     = @_;
    my $maint_user = "";
    my $rc         = 0;
    my $password   = "";

    open( MEMBERS, "/usr/bin/members maintenance |" )  or die "Failed to execute members";
    $maint_user = Text::trim(<MEMBERS>);
    close(MEMBERS);

    print "\nChanging password for user: $maint_user\n\n";

	$password = readMaintPasswordFromOVF();
    
    # Commit Change
    system( "/usr/bin/chattr", "-i", "/etc/passwd", "/etc/shadow" );
    my $pu = Passwd::Unix->new();
    if ( $pu->passwd( $maint_user, $pu->encpass($password) ) ) {
        print "\nPassword set successfully.\n";
    }
    else {
        print "\nError updating password.\n";
        $rc = 1;
    }
    system( "/usr/bin/chattr", "+i", "/etc/passwd", "/etc/shadow" );

    return $rc;
}

sub validateText {
    my $text = $_[0];
    my $min_len = 8;
    if ( length($text) < $min_len ) {
        return 0;
    }
    return 1;
}

##################################################################################################################
# Main execution
##################################################################################################################
# At boot, the appliance's startup messages are supressed by redirecting them to an alternate console /dev/ttyS0
# For the interactive portions of this program, we'll need to explicitly switch STDIN and STDOUT to use the real console at /dev/tty1
sub main() {
    open( STDIN,  "</dev/tty1" );
    open( STDOUT, ">/dev/tty1" );

    my $mode = processArguments();
    my $rc   = 0;

    # Clear the screen
    #print (((" " x 80) . "\n") x 25);
    system("clear");

    if ($mode eq "check") {
        # We don't do anything for -check
        exit($rc);
    }

    # Everything else sets maintenance password
    print "\nSet MAINTENANCE user password\n\n";
    if ( $mode eq "init" ) {
        # Read OVF environment
        $rc = doConfig();
		if ($rc == 0) {
            exit(0);
        } else {
            print "\n";
            print "No MAINTENANCE user password was retrieved during OVF prompts.\n";
            print "This could be due to either an issue with VMware Tools or\n";
            print "network configuration issue.\n";
            print "\n";
            
            doConfig();
        }
    }    #End init
    elsif( $mode eq "config" ) {
        doConfig();
    }  # End config
        
    exit(0);
}

main;

