#!/usr/bin/perl

# $Id:  $
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.


use strict;
use Getopt::Long;
use Pod::Usage;
use Switch;

use File::Basename;
# add the current dirname to the lib path so we can find our other modules
use lib dirname(__FILE__);
use NetConfig::Config;

=head1 NAME

network-config - Configure networking on this host.

=head1 SYNOPSIS

network-config <command> <subcommand> [options] [arguments]

=head1 DESCRIPTION

Displays and configures all aspects of networking on this host, including:

    - hostname
    - static and dhcp ip configuration
    - dns nameservers / domain / search domains
    - static routes

Piecemeal changes can be made by beginning an edit session (B<edit begin>),
making changes referencing the edit session id (sid),
and then committing the edit session (B<edit commit>).

Alternatively, complete reconfiguration can be done using the B<config set> command.

This command is meant to be used internally by

    - first boot scripts that prompt the user for networking configuration
    - first boot scripts that configure networking from config files
    - the maintenance console
    - the web ui

The maintenance console uses edit sessions.  The other usages will normally use B<config set>.

Changes are made to the F</etc/network/interfaces>, F</etc/hostname>, and F</etc/hosts> files.

For any list/get commands, if no session id (sid) is provided,
information from the active system is returned.
If a sid is provided, then information from the edit session is returned.

=head1 COMMANDS and SUBCOMMANDS

=over 4

=item B<edit begin>

Begins an edit session.
Prints a unique session id (sid) string on stdout that must be used for all edits in this session.
Changes made during this edit session do not take effect until they are committed using B<edit commit>.
To rollback the changes, use B<edit rollback>.

Internally, when an edit session starts, configuration files are copied into a temporary location.
During the lifetime of the edit session, modifications are performed on these temporary files,
and do not affect the live system.  Also, copies of the original files are maintained in order
to detect changes made outside of this edit session.

=cut
sub cmd_edit_begin {
    verify_no_more_arguments();
    my $sid = edit_begin();
    print "$sid\n";
    return 0;
}

=item B<edit commit --session E<lt>sidE<gt> [--skip-refresh] [--preview]>

Commits the edit session identified with <sid> (or previews the commit if --preview is specified).

If --preview is not provided, then the copies of the original files are compared
against the original files. If the original files have not changed since this edit
session started, then the edited files are copied to the original locations,
and the live system is refreshed to reflect the new configuration.

If the original files have changed (indicating some other process has changed them since
the edit session started), then the commit fails and the edit session rolled back.
Exit code 2 is returned.

If --skip-refresh is provided the live system will need to be refreshed externally
(e.g. networking restarted, etc) otherwise the live system will be refreshed with
the changes immediately.

If --preview is provided, then the commit will not actually occur.  An exit code of 0
indicates if the commit should succeed.  Exit code 2 is returned if some other process
has changed the live files after the edit session started.

=cut
sub cmd_edit_commit {
    my $sid;
    my $skipRefresh = undef;
    my $previewOnly = undef;
    GetOptions(
        "session=s"   => \$sid,
        "skip-refresh"   => \$skipRefresh,
        "preview"   => \$previewOnly
    ) or print_usage_error();
    verify_required_options("session" => $sid);
    verify_no_more_arguments();
    my $commitStatus = edit_commit($sid, $skipRefresh, $previewOnly);
    if ($commitStatus == 2) {
        if ($previewOnly) {
            print "Original configuration has changed since this edit session was started. Commit will fail.\n";
        } else {
            print "Original configuration has changed since this edit session was started. Edit session has been rolled back.\n";
        }
    }
    return $commitStatus;
}

=item B<edit rollback --session E<lt>sidE<gt>>

Reverts the edit session identified with <sid>.

=cut
sub cmd_edit_rollback {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_required_options("session" => $sid);
    verify_no_more_arguments();
    edit_rollback($sid);
    return 0;
}

=item B<edit status --session E<lt>sidE<gt>>

Determines if edits have been made in this edit session
Exits with 0 if no edits have been made in this edit session
Exits with 2 if edits have been made in this edit session
Exits with 3 if edit session does not exist

=cut
sub cmd_edit_status {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_required_options("session" => $sid);
    verify_no_more_arguments();
    my $status = edit_status($sid);
    switch ($status) {
        case 0 {
            print "No edits have been made in this edit session.\n";
        }
        case 2 {
            print "Edits have been made in this edit session.\n";
        }
        case 3 {
            print "Edit session does not exist.\n";
        }
    }
    return $status;
}

=item B<interface list>

Lists all (non-loopback) interface names available on the system on stdout one per line.

=cut
sub cmd_interface_list {
    verify_no_more_arguments();
    my @interfaces = interface_list();
    print join("\n", @interfaces) . "\n";
    return 0;
}

=item B<interface get [--session E<lt>sidE<gt>] E<lt>interfaceE<gt>>

Prints information about the given interface on stdout.
If no sid is provided, information from the active system is returned.
If a sid is provided, then information from the edit session is returned.

The information returned is in the following format (not necessarily in this order):

    NAME=<interface name>
    STATUS=<ENABLED|DISABLED>
    MODE=<STATIC|DHCP>
    ADDRESS=<ip address>
    MASK=<network mask>
    GATEWAY=<gateway>
    BROADCAST=<broadcast>
    NETWORK=<network>

If a session is provided and C<MODE=DHCP>, then C<ADDRESS>, C<MASK>, C<GATEWAY>, C<BROADCAST>, and C<NETWORK> will not be returned.

=cut
sub cmd_interface_get {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    my $interfaceName = shift @ARGV;
    verify_required_arguments("interface" => $interfaceName);
    verify_no_more_arguments();
    my $interface = interface_get($sid, $interfaceName);
    print_hash_with_flat_arrays($interface);
    return 0;
}


###############################################################################
# Find the network ethernet interface
###############################################################################
sub getEthernetInterface() {
     # Find the first ethernet interface
     my $interfaceName = "eth0";
     open(FH, "/bin/ls -1 /sys/class/net |") or die "Could not run /bin/ls: $?\n";
     while(<FH>) {
          chomp;
          if ($_ =~ /^(e.*)/) {
              $interfaceName = $1;
              last;
          }
     }
     close(FH);
     return $interfaceName;
}

my $ETHERNET_INTERFACE = getEthernetInterface();


=item B<interface discoverIps>

Return values:

    IP4=<IPv4 address>
    IP6=<IPv6 adress>

=cut
sub cmd_interface_discoverIps {

    # my $ETHERNET_INTERFACE ="eth0";

    my $IP_PATTERN = "((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))";

    my $IP6_PATTERN = qr/^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$/;


     # Execute the ifconfig eth0 command to search for the IP4 address
       open(IFCONFIG, "ifconfig $ETHERNET_INTERFACE|") or die "Failed to execute ifconfig";
       my $ip4Address = "";
       while (<IFCONFIG>) {
          # Look for the IPV4 address
          if ($_ =~ /inet $IP_PATTERN/) {
              $ip4Address = $1;
              last;
          }
       }
       close(IFCONFIG);

       # Execute the ifconfig eth0 command to search for the IP6 address
       open(IFCONFIG, "/sbin/ifconfig $ETHERNET_INTERFACE |grep 'inet6'|") or die "Failed to execute ifconfig";
       my $ip6Address = "";
       while (<IFCONFIG>) {
          # Look for the IPv6 address
          if ($_ =~ /inet6/) {
              my @temp = split /\s+/, $_;
              if ($temp[6] =~ m/<global>/) {
                  $ip6Address = $temp[2];
                  last;       
              }
          }
       }
       close(IFCONFIG);

    my $iphash = ({IP4 => $ip4Address, IP6 => $ip6Address});
    print_hash_with_flat_arrays($iphash);
    return 0;
}


=item B<interface enable --session E<lt>sidE<gt> E<lt>interfaceE<gt>>

Enables the given interface (so that it is started automatically).

=cut
sub cmd_interface_enable {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_required_options("session" => $sid);
    my $interfaceName = shift @ARGV;
    verify_required_arguments("interface" => $interfaceName);
    verify_no_more_arguments();
    interface_enable($sid, $interfaceName);
    return 0;
}

=item B<interface disable --session E<lt>sidE<gt> E<lt>interfaceE<gt>>

Disables the given interface (so that it is not started automatically).

=cut
sub cmd_interface_disable {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_required_options("session" => $sid);
    my $interfaceName = shift @ARGV;
    verify_required_arguments("interface" => $interfaceName);
    verify_no_more_arguments();
    interface_disable($sid, $interfaceName);
    return 0;
}

=item B<interface set-static --session E<lt>sidE<gt> --address E<lt>addressE<gt> --mask E<lt>maskE<gt> --gateway E<lt>gatewayE<gt> E<lt>interfaceE<gt>>

Configures the given interface to use the given static network configuration.

=cut
sub cmd_interface_set_static {
    my ($sid, $address, $mask, $gateway);
    GetOptions(
        "session=s"   => \$sid,
        "address=s"   => \$address,
        "mask=s"      => \$mask,
        "gateway=s"   => \$gateway,
    ) or print_usage_error();
    verify_required_options(
        "session"     => $sid,
        "address"     => $address,
        "mask"        => $mask,
        "gateway"     => $gateway,
    );
    my $interfaceName = shift @ARGV;
    verify_required_arguments("interface" => $interfaceName);
    verify_no_more_arguments();

    interface_set_static(
        $sid,
        $interfaceName,
        {
            ADDRESSFAMILY => "IPv4",
            ADDRESS   => $address,
            MASK      => $mask,
            GATEWAY   => $gateway,
        }
    );
    return 0;
}

################################################
#This sets static network configuration 
#when user wants to swtich to IPV6 network settings
################################################

=item B<interface set-static-ipv6 --session E<lt>sidE<gt> --address E<lt>addressE<gt> --prefixlength E<lt>prefixlengthE<gt> --gateway E<lt>gatewayE<gt> E<lt>interfaceE<gt>>

Configures the given interface to use the given static network configuration.

=cut
sub cmd_interface_set_static_ipv6 {
    my ($sid, $address, $prefixlength, $gateway, $network);
    GetOptions(
        "session=s"   => \$sid,
        "address=s"   => \$address,
        "prefixlength=s"      => \$prefixlength,
        "gateway=s"   => \$gateway,
        "network=s"    => \$network,
    ) or print_usage_error();
    verify_required_options(
        "session"     => $sid,
        "address"     => $address,
        "prefixlength"        => $prefixlength,
        "gateway"     => $gateway,
    );
    my $interfaceName = shift @ARGV;

    verify_required_arguments("interface" => $interfaceName);
    verify_no_more_arguments();
    interface_set_static(
        $sid,
        $interfaceName,
        {
            ADDRESSFAMILY => "IPv6",
            ADDRESS   => $address,
            PREFIXLENGTH  => $prefixlength,
            GATEWAY   => $gateway,
            NETWORK => $network,
        }
    );
    return 0;
}



=item B<interface set-dhcp --session E<lt>sidE<gt> E<lt>interfaceE<gt>>

Configures the given interface to use dhcp network configuration.

=cut
sub cmd_interface_set_dhcp {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_required_options("session" => $sid);
    my $interfaceName = shift @ARGV;
    verify_required_arguments("interface" => $interfaceName);
    verify_no_more_arguments();
    interface_set_dhcp($sid, $interfaceName);
    return 0;
}

=item B<dns set --session E<lt>sidE<gt> [--nameserver E<lt>nameserver1,nameserver2,...E<gt>] [--domain E<lt>domainE<gt>] [--search E<lt>searchdomain1,searchdomain2,...E<gt>]>

Configures the given dns settings.
If dhcp is used by any interface, then these settings will be in addition to the ones returned by dns.

=cut
sub cmd_dns_set {
    my ($sid, $addressfamily, $nameserver, $domain, $search);
    GetOptions(
        "session=s"    => \$sid,
        "addressfamily=s" => \$addressfamily,
        "nameserver=s" => \$nameserver,
        "domain=s"     => \$domain,
        "search=s"     => \$search,
    ) or print_usage_error();
    verify_required_options(
        "session"      => $sid,
        "addressfamily" => $addressfamily
    );
    verify_at_least_one_option(
        "nameserver"   => $nameserver,
        "domain"       => $domain,
        "search"       => $search,
    );
    verify_no_more_arguments();

    my %args;

    if (defined $nameserver) {
        verify_not_blank_arguments(
            "nameserver" => $nameserver
        );
        my @nameservers = split(",", $nameserver);
        $args{NAMESERVERS} = \@nameservers;
    }

    if (defined $domain) {
        $args{DOMAIN} = $domain;
    }

    if (defined $search) {
        my @searchDomains = split(",", $search);
        $args{SEARCHDOMAINS} = \@searchDomains;
    }

    dns_set(
        $sid,
        $addressfamily,
        \%args
    );
    return 0;
}

=item B<dns get [--session E<lt>sidE<gt>]>

Prints information about dns configuration on stdout.
If no sid is provided, information from the active system is returned.
If a sid is provided, then information from the edit session is returned.

The information returned is in the following format:

    NAMESERVERS=<nameserver1,nameserver2,...>
    DOMAIN=<domain name>
    SEARCHDOMAINS=<searchdomain1,searchdomain2,...>

=cut
sub cmd_dns_get {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_no_more_arguments();
    my $dns = dns_get($sid);
    print_hash_with_flat_arrays($dns);
    return 0;
}

=item B<route list [--session E<lt>sidE<gt>] --interface E<lt>interfaceE<gt>>

Prints information about routes configured for the given interface
If no sid is provided, information from the active system is returned.
If a sid is provided, then information from the edit session is returned.

One (network or host) route destination is printed on each line in the form:

    <network-or-host-address> <gateway>

For example:

    172.20.11.0/16 172.20.10.254
    172.20.11.10 172.20.10.254

=cut
sub cmd_route_list {
    my ($sid, $interfaceName);
    GetOptions(
        "session=s"   => \$sid,
        "interface=s"  => \$interfaceName,
    ) or print_usage_error();
    verify_required_options("interface" => $interfaceName);
    verify_no_more_arguments();
    my @routes = route_list($sid, $interfaceName);
    if (@routes) {
        print join("\n", map { "$_->{DESTINATION} $_->{GATEWAY}" } @routes) . "\n";
    }
    return 0;
}

=item B<route add --session E<lt>sidE<gt> --interface E<lt>interfaceE<gt> --destination E<lt>network-or-host-addressE<gt> --gateway E<lt>gatewayE<gt>>

Add a static route to the given network or host to the given interface.
For networks, specify the number of bits to mask at the end of the network address like this: 192.168.45.0/16

=cut
sub cmd_route_add {
    my ($sid, $interfaceName, $destination, $gateway);
    GetOptions(
        "session=s"      => \$sid,
        "interface=s"    => \$interfaceName,
        "destination=s"  => \$destination,
        "gateway=s"      => \$gateway,
    ) or print_usage_error();
    verify_required_options(
        "session"        => $sid,
        "interface"      => $interfaceName,
        "destination"    => $destination,
        "gateway"        => $gateway,
    );

    verify_no_more_arguments();

    route_add(
        $sid,
        $interfaceName,
        {
            DESTINATION => $destination,
            GATEWAY => $gateway,
        }
    );
}

=item B<route delete --session E<lt>sidE<gt> --interface E<lt>interfaceE<gt> --destination E<lt>network-or-host-addressE<gt> --gateway E<lt>gatewayE<gt>>

Removes a static route to the given network or host from the given interface.
For networks, specify the number of bits to mask at the end of the network address like this: 192.168.45.0/16

=cut
sub cmd_route_delete {
    my ($sid, $interfaceName, $destination, $gateway);
    GetOptions(
        "session=s"      => \$sid,
        "interface=s"    => \$interfaceName,
        "destination=s"  => \$destination,
        "gateway=s"      => \$gateway,
    ) or print_usage_error();
    verify_required_options(
        "session"        => $sid,
        "interface"      => $interfaceName,
        "destination"    => $destination,
        "gateway"        => $gateway,
    );

    verify_no_more_arguments();

    route_delete(
        $sid,
        $interfaceName,
        {
            DESTINATION => $destination,
            GATEWAY => $gateway,
        }
    );
    return 0;
}

=item B<hostname get [--session E<lt>sidE<gt>]>

Prints the name of this host on stdout.
If no sid is provided, information from the active system is returned.
If a sid is provided, then information from the edit session is returned.

=cut
sub cmd_hostname_get {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_no_more_arguments();
    my $hostname = hostname_get($sid);
    print "$hostname\n";
    return 0;
}

=item B<hostname set --session E<lt>sidE<gt> E<lt>hostnameE<gt>>

Sets the unqualified name of this host.  (i.e. no domain name)

The hostname must be less than 64 characters.  Only letters a-z (case insensitive), digits 0-9, and the hyphen (-) are allowed.  The hostname must not start with a hyphen.

=cut
sub cmd_hostname_set {
    my ($sid, $hostname);
    GetOptions(
        "session=s"    => \$sid,
    ) or print_usage_error();
    verify_required_options(
        "session"      => $sid,
    );
    my $hostname = shift @ARGV;
    verify_required_arguments("hostname" => $hostname);
    verify_no_more_arguments();
    hostname_set(
        $sid,
        $hostname
    );
    return 0;
}

=item B<config get [--session E<lt>sidE<gt>]>

Prints all configuration information on stdout.
If no sid is provided, information from the active system is returned.
If a sid is provided, then information from the edit session is returned.

The information returned is in the following format (but not necessarily in this order, or with the same spacing):

    HOSTNAME=<host name>

    INTERFACES=<interfacename1,interfacename2,...>

    INTERFACE_<interfacename1>_NAME=<interface name>
    INTERFACE_<interfacename1>_STATUS=<ENABLED|DISABLED>
    INTERFACE_<interfacename1>_MODE=<STATIC|DHCP>
    INTERFACE_<interfacename1>_ADDRESS=<ip address>
    INTERFACE_<interfacename1>_MASK=<network mask>
    INTERFACE_<interfacename1>_GATEWAY=<gateway>
    INTERFACE_<interfacename1>_BROADCAST=<broadcast>
    INTERFACE_<interfacename1>_NETWORK=<network>
    INTERFACE_<interfacename1>_ROUTES=<network-or-host-address> <gateway>,<network-or-host-address> <gateway>,...

    INTERFACE_<interfacename2>_NAME=<interface name>
    INTERFACE_<interfacename2>_STATUS=<ENABLED|DISABLED>
    INTERFACE_<interfacename2>_MODE=<STATIC|DHCP>
    INTERFACE_<interfacename2>_ADDRESS=<ip address>
    INTERFACE_<interfacename2>_MASK=<network mask>
    INTERFACE_<interfacename2>_GATEWAY=<gateway>
    INTERFACE_<interfacename2>_BROADCAST=<broadcast>
    INTERFACE_<interfacename2>_NETWORK=<network>
    INTERFACE_<interfacename2>_ROUTES=<network-or-host-address> <gateway>,<network-or-host-address> <gateway>,...

    NAMESERVERS=<nameserver1,nameserver2,...>
    DOMAIN=<domain name>
    SEARCHDOMAINS=<searchdomain1,searchdomain2,...>

=cut
sub cmd_config_get {
    my $sid;
    GetOptions("session=s"   => \$sid) or print_usage_error();
    verify_no_more_arguments();
    my $config = config_get($sid);
    print_hash_with_flat_arrays($config);
    return 0;
}

=item B<config set [--skip-refresh]>

Configures the system with configuration information read from stdin.
The configuration information must be in the form as returned by B<config get>.

No edit session is needed for this command.  Changes are persisted immediately.
If --skip-refresh is NOT provided then the live system will be refreshed with the changes immediately,
otherwise the live system will need to be refreshed externally (e.g. networking restarted, etc).

For a given interface, if C<MODE=DHCP>, then C<ADDRESS>, C<MASK>, and C<GATEWAY> do not need to be provided as input.

The C<BROADCAST> and C<NETWORK> do not need to be provided as input, and will be ignored.

Not all fields need to be specified.  If a field is not present it will not be modified.

To clear the value of a field, include the field name and the equals sign, but do not provide a value.  For example: C<INTERFACE_eth0_ROUTES=>

=cut
sub cmd_config_set {
    my $skipRefresh = undef;
    GetOptions("skip-refresh"   => \$skipRefresh) or print_usage_error();
    verify_no_more_arguments();

    # Read in the properties from STDIN
    my %config = map {
       s/#.*//;
       s/^\s+//;
       s/\s+$//;
       m/(.*?)\s*=\s*(.*)/;
    } <STDIN>;

    # Convert comma separated values to lists for specific entries
    split_value(\%config, "INTERFACES");
    split_value(\%config, "NAMESERVERS");
    split_value(\%config, "SEARCHDOMAINS");

    if (defined $config{"INTERFACES"}) {
        foreach my $interfaceName ( @{$config{"INTERFACES"}} ) {
            split_value(\%config, "INTERFACE_${interfaceName}_ROUTES");
        }
    }

    # now set the config
    config_set(\%config, $skipRefresh);
    return 0;
}

sub split_value {
    my ($hash_ref, $key) = @_;
    if (defined $hash_ref->{$key}) {
        my @values = split(',', $hash_ref->{$key});
        $hash_ref->{$key} = \@values;
    }
}

=item B<config validate [--interface E<lt>interfaceE<gt>]>

Tests the current live configuration of the given interface to determine if it is valid (by pinging the gateway).
If the interface name is not specified, then defaults to eth0.
If the configuration is valid, a zero exit code is returned.
If the configuration is invalid, the reason why it is invalid is printed on stdout, and a non-zero exit code is returned.

=cut
sub cmd_config_validate {
    my ($interfaceName);
    GetOptions(
        "interface=s"    => \$interfaceName,
    ) or print_usage_error();
    verify_no_more_arguments();
    my $errorMessage = config_validate($interfaceName);
    if (defined $errorMessage) {
        print STDERR "$errorMessage\n";
        return 1;
    }
    return 0;
}

=item B<help>

Print this help information

=cut
sub cmd_help_ {
    pod2usage(-verbose  => 99, -exitval => 0);
    return 0;
}

=back

=cut


sub print_hash_with_flat_arrays {
    my ($hash_ref) = @_;
    foreach my $key ( sort keys %$hash_ref ) {
        my $value = $hash_ref->{$key};
        if(ref($value) eq 'ARRAY') {
            print "$key=" . join(",", @{$value}) . "\n";
        } else {
            print "$key=$value\n";
        }
    }
}

sub verify_required_options {
    my %required_options = @_;
    foreach my $option_name ( keys %required_options ) {
        if (!defined($required_options{$option_name})) {
            print_usage_error("You must specify the --$option_name option");
        }
    }
}

sub verify_at_least_one_option {
    my %required_options = @_;
    foreach my $option_name ( keys %required_options ) {
        if (defined($required_options{$option_name})) {
            return;
        }
    }
    print_usage_error("You must specify at least one of " . join(", ", map { "--$_" } keys %required_options));
}

sub verify_required_arguments {
    my %required_arguments = @_;
    foreach my $argument_name ( keys %required_arguments ) {
        if (!defined($required_arguments{$argument_name})) {
            print_usage_error("You must specify the $argument_name argument");
        }
    }
}

sub verify_not_blank_arguments {
    my %required_arguments = @_;
    foreach my $argument_name ( keys %required_arguments ) {
        if (!defined($required_arguments{$argument_name}) or $required_arguments{$argument_name} =~ /^\s*$/) {
            print_usage_error("You must specify the $argument_name argument");
        }
    }
}

sub verify_no_more_arguments {
    if (@ARGV != 0) {
        print_usage_error("Unknown arguments: " . join(', ', @ARGV));
    }
}

sub print_usage_error {
    my ($message) = @_;
    pod2usage(-verbose  => 99, -exitval => 1, -message => "$message")
}


my $command = shift;
my $subcommand = shift;

my $command_handler_name = "cmd_${command}_${subcommand}";
$command_handler_name =~ s/-/_/g;
if (defined(&$command_handler_name)) {
    exit &{ \&$command_handler_name }();
} else {
    print_usage_error("Unknown command: $command $subcommand");
}



